from openfisca_france.model.base import Variable, Individu, MONTH, set_input_divide_by_period


class allocation_annuelle_etudiant(Variable):
    value_type = float
    entity = Individu
    label = "Allocation annuelle (type d'aide spécifique aux étudiants à difficultés pérennes)"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'https://www.education.gouv.fr/bo/14/Hebdo40/MENS1420893C.htm',
        'https://www.etudiant.gouv.fr/fr/aides-specifiques-1306'
        ]
    documentation = '''
    Cette aide est l'équivalent d'une bourse sur critères sociaux.
    Elle permet ainsi d'être exonéré·e des droits d'inscription universitaires
    et de la contribution vie étudiante et de campus (CVEC).
    '''



from openfisca_france.model.base import *
from openfisca_core import periods
from numpy import logical_and as and_, logical_not as not_, logical_or as or_


class eligibilite_per(Variable):
    entity = Famille
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000046289935'
    label = 'Eligibilité à la prime exceptionnelle de rentrée'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2022(famille, period):

        juin_2022 = periods.period('2022-06')

        eligibilite = famille('rsa', juin_2022)
        + famille('apl', juin_2022)
        + famille('alf', juin_2022)
        + famille('als', juin_2022)
        + famille('aspa', juin_2022)

        eligibilite_ass_i = famille.members('ass', juin_2022)
        eligibilite_ass = famille.sum(eligibilite_ass_i)

        eligibilite_aer_i = famille.members('aer', juin_2022)
        eligibilite_aer = famille.sum(eligibilite_aer_i)

        eligibilite_aah_i = famille.members('aah', juin_2022)
        eligibilite_aah = famille.sum(eligibilite_aah_i)

        eligibilite_pfmra_i = famille.members('prime_forfaitaire_mensuelle_reprise_activite', juin_2022)
        eligibilite_pfmra = famille.sum(eligibilite_pfmra_i)

        return (eligibilite + eligibilite_ass + eligibilite_aer + eligibilite_aah + eligibilite_pfmra) > 0


class eligibilite_per_etudiant(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000046289843'
    label = 'Eligibilité à la prime exceptionnelle de rentrée des étudiants boursiers'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2022(individu, period):
        juin_2022 = periods.period('2022-06')
        eligibilite_etudiant = where(
            not_(or_(individu('bourse_criteres_sociaux', juin_2022) > 0, individu('bourse_enseignement_sup', juin_2022) > 0)),
            False,
            where(
                and_(individu.has_role(Famille.PARENT), individu.famille('eligibilite_per', period)),
                False,
                True
                )
            )

        return eligibilite_etudiant


class eligibilite_per_ppa(Variable):
    entity = Famille
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000046556491'
    label = 'Eligibilité à la prime exceptionnelle de rentrée des bénéficiaires de la prime d activité'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2022(famille, period):
        juin_2022 = periods.period('2022-06')
        eligibilite_ppa = (famille('ppa', juin_2022) > 0) * (not_(famille('eligibilite_per', period)))

        return eligibilite_ppa


class prime_exceptionnelle_rentree_non_etudiant(Variable):
    entity = Famille
    value_type = float
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000046289935'
    label = 'Prime exceptionnelle de rentrée'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2022(famille, period, parameters):
        '''
        Hypothèse : Vu qu'on suppose qu'un jeune boursier vivant chez ses parents peut aussi toucher la PER étudiant, même si ses parents
                    touchent la PER aussi (voir la docstring de la variable prime_exceptionnelle_rentree_etudiant), on retire ces enfants-là
                    des enfants à charge.
        '''

        juin_2022 = periods.period('2022-06')
        prestations_familiales_enfant_a_charge_i = famille.members('prestations_familiales_enfant_a_charge', juin_2022)
        eligibilite_per_etudiant_i = famille.members('eligibilite_per_etudiant', period)
        nb_enfants = famille.sum(prestations_familiales_enfant_a_charge_i * (1 - eligibilite_per_etudiant_i))
        parametres_per = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.prime_exceptionnelle_rentree

        per = where(
            famille('eligibilite_per', period),
            parametres_per.per + nb_enfants * parametres_per.per_enfant,
            where(
                famille('eligibilite_per_ppa', period),
                parametres_per.per_ppa + nb_enfants * parametres_per.per_ppa_enfant,
                0
                )
            )

        return per


class prime_exceptionnelle_rentree_etudiant(Variable):
    entity = Individu
    value_type = float
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000046289843'
    label = 'Prime exceptionnelle de rentrée pour les étudiants boursiers'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2022(individu, period, parameters):
        '''
        Hypothèse 1 : Si l'étudiant a le rôle enfant dans son entité famille, ses enfants ne sont pas identifiés. La majoration pour enfant
                      n'entre donc pas en compte dans le calcul.
        Hypothèse 2 : Si un jeune est boursier et vit chez ses parents, il touche la PER étudiant, même si ses parents touchent aussi la PER.
                      Il n'a pas été trouvé de réponse claire à ce cas de figure dans la loi. D'où le fait de faire une hypothèse ici.
        '''

        juin_2022 = periods.period('2022-06')
        prestations_familiales_enfant_a_charge_i = individu.famille.members('prestations_familiales_enfant_a_charge', juin_2022)
        nb_enfants = individu.famille.sum(prestations_familiales_enfant_a_charge_i)
        parametres_per = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.prime_exceptionnelle_rentree

        per_etudiant = where(
            not_(individu('eligibilite_per_etudiant', period)),
            0,
            where(
                individu.has_role(Famille.ENFANT),
                parametres_per.per_etudiant,
                parametres_per.per_etudiant + nb_enfants * parametres_per.per_etudiant_enfant
                )
            )

        return per_etudiant



#! /usr/bin/env python

import os
import re
import tokenize


DOUBLE_QUOTE_DOCSTRING_RE = re.compile(
    r'''^(?P<prefix>[furbFURB]*)"""(?P<content>.*?)"""$''',
    re.DOTALL)
DOUBLE_QUOTE_STRING_RE = re.compile(
    r'''^(?P<prefix>[furbFURB]*)"(?P<content>[^']*)"$''')
# SINGLE_QUOTE_DOCSTRING_RE = re.compile(
#     r"""^(?P<prefix>[furbFURB]*)'''(?P<content>.*?)'''$""",
#     re.DOTALL)
# SINGLE_QUOTE_STRING_RE = re.compile(
#     r"""^(?P<prefix>[furbFURB]*)'(?P<content>[^"]*)'$""")


def iterate_tokens(file_path):
    with open(file_path, encoding='utf-8') as python_file:
        for token in tokenize.generate_tokens(python_file.readline):
            if token.type == tokenize.STRING:
                token = token._replace(string=DOUBLE_QUOTE_DOCSTRING_RE.sub(
                    "\g<prefix>'''\g<content>'''",  # noqa: W605
                    token.string))
                token = token._replace(string=DOUBLE_QUOTE_STRING_RE.sub(
                    "\g<prefix>'\g<content>'",  # noqa: W605
                    token.string))
            yield token


project_dir = os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.abspath(__file__))))
for path, dirs, files in os.walk(project_dir):
    for dir in dirs:
        if dir.startswith('.') and dir != '.github':
            dirs.remove(dir)
    for file in files:
        if file.startswith('.') or not file.endswith('.py'):
            continue
        file_path = os.path.join(path, file)
        normalized_source_code = tokenize.untokenize(iterate_tokens(file_path))
        with open(file_path, 'w', encoding='utf-8') as python_file:
            python_file.write(normalized_source_code)



from numpy import select, where, logical_not as not_
from openfisca_france.model.base import (
    Enum,
    Variable,
    Individu,
    Famille,
    MONTH,
    set_input_divide_by_period,
    )


class css_participation_forfaitaire_montant_i(Variable):
    value_type = float
    entity = Individu
    label = "Montant de la participation forfaitaire d'une personne"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.css.montant
        age = individu('age', period)
        salarie_regime_alsace_moselle = individu('salarie_regime_alsace_moselle', period)
        regime = where(
            salarie_regime_alsace_moselle,
            RegimeComplementaireSanteSolidaire.alsace_moselle,
            RegimeComplementaireSanteSolidaire.france,
            )
        tranche = select(
            [
                age < 30,
                age <= 49,
                age <= 59,
                age <= 69,
                age > 69,
                ],
            [
                TranchesComplementaireSanteSolidaire.cmu_moins_30_ans,
                TranchesComplementaireSanteSolidaire.cmu_30_49_ans,
                TranchesComplementaireSanteSolidaire.cmu_50_59_ans,
                TranchesComplementaireSanteSolidaire.cmu_60_69_ans,
                TranchesComplementaireSanteSolidaire.cmu_plus_69_ans,
                ],
            )
        return P[regime][tranche]


class css_participation_forfaitaire_montant(Variable):
    value_type = float
    entity = Famille
    label = "Montant de participation forfaitaire d'une famille en cas d'éligibilité"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        css_participation_forfaitaire_i = famille.members('css_participation_forfaitaire_montant_i', period)
        return famille.sum(css_participation_forfaitaire_i)


class css_participation_forfaitaire(Variable):
    value_type = float
    label = 'Montant annuel de la participation forfaitaire à la CSS'
    entity = Famille
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_11_01(famille, period):
        cmu_c = famille('cmu_c', period)
        css_cmu_base_ressources = famille('css_cmu_base_ressources', period)
        css_plafond = famille('acs_plafond', period)
        css_participation_forfaitaire_montant = famille('css_participation_forfaitaire_montant', period)
        residence_mayotte = famille.demandeur.menage('residence_mayotte', period)
        css_cmu_acs_eligibilite = famille('css_cmu_acs_eligibilite', period)
        acs = famille('acs', period)

        return (
            css_cmu_acs_eligibilite
            * not_(residence_mayotte)
            * not_(cmu_c)
            * (acs == 0)
            * (css_cmu_base_ressources <= css_plafond)
            * css_participation_forfaitaire_montant
            )


class RegimeComplementaireSanteSolidaire(Enum):
    france = 'France'
    alsace_moselle = 'Alsace Moselle'


class TranchesComplementaireSanteSolidaire(Enum):
    cmu_moins_30_ans = 'Moins de 30 ans'
    cmu_30_49_ans = 'Entre 30 et 49 ans'
    cmu_50_59_ans = 'Entre 50 et 59 ans'
    cmu_60_69_ans = 'Entre 60 et 69 ans'
    cmu_plus_69_ans = 'Plus de 69 ans'



from openfisca_france.model.base import *


class TypesEligibiliteANAH(Enum):
    # Needed to preserve the enum order in Python 2
    __order__ = 'a_verifier modestes tres_modeste'
    a_verifier = 'A vérifier'
    modestes = 'Modestes'
    tres_modeste = 'Très modestes'


class eligibilite_anah(Variable):
    value_type = Enum
    possible_values = TypesEligibiliteANAH
    default_value = TypesEligibiliteANAH.a_verifier
    entity = Menage
    label = "Barème d'éligibilité aux aides ANAH"
    definition_period = YEAR

    def formula(menage, period):
        in_idf = menage('residence_ile_de_france', period.first_month)

        rfr_declarants_principaux_du_menage = menage.members.has_role(FoyerFiscal.DECLARANT_PRINCIPAL) * menage.members.foyer_fiscal('rfr', period.n_2)
        rfr = menage.sum(rfr_declarants_principaux_du_menage)

        nb_members = menage.nb_persons()

        bareme_idf = select(
            [nb_members == 1, nb_members == 2, nb_members == 3, nb_members == 4, nb_members >= 5],
            [select([rfr <= 19875, rfr <= 24194], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 29171, rfr <= 35510], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 35032, rfr <= 42648], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 40905, rfr <= 49799], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 46798 + ((nb_members - 5) * 5882), rfr <= 56970 + ((nb_members - 5) * 7162)], [2, 1], 0)])

        bareme_out = select(
            [nb_members == 1, nb_members == 2, nb_members == 3, nb_members == 4, nb_members >= 5],
            [select([rfr <= 14360, rfr <= 18409], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 21001, rfr <= 26923], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 25257, rfr <= 32377], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 29506, rfr <= 37826], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier),
             select([rfr <= 33774 + ((nb_members - 5) * 4257), rfr <= 43297 + ((nb_members - 5) * 5454)], [TypesEligibiliteANAH.tres_modeste, TypesEligibiliteANAH.modestes], TypesEligibiliteANAH.a_verifier)])

        return where(in_idf, bareme_idf, bareme_out)



from openfisca_france.model.base import *


class retraite_brute(Variable):
    value_type = float
    entity = Individu
    label = 'Retraite brute'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add
    unit = 'currency'


class aer(Variable):
    value_type = int
    entity = Individu
    label = 'Allocation équivalent retraite (AER)'
    # L'AER est remplacée depuis le 1er juillet 2011 par l'allocation transitoire de solidarité (ATS).
    definition_period = MONTH
    set_input = set_input_divide_by_period
    unit = 'currency'


class retraite_combattant(Variable):
    value_type = float
    entity = Individu
    label = 'Retraite du combattant'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    unit = 'currency'



'''
This files tests the performance of the test runner of openfisca-run-test on a subset of YAML tests.
It is placed in openfisca-france because it is the largest set we currently have.

Usage example:

    python openfisca_france/scripts/performance/measure_tests_performance.py
'''

import os
import time
import logging
import pkg_resources

from openfisca_core.tools.test_runner import run_tests
from openfisca_france import CountryTaxBenefitSystem

# Create logger
logging.basicConfig(level = logging.INFO)
logger = logging.getLogger(__name__)

# Baselines for comparision - unit : seconds
BASELINE_TBS_LOAD_TIME = 9.10831403732
BASELINE_YAML_TESTS_TIME = 271.448431969


# Time tax benefit system loading
start_time_tbs = time.time()
tbs = CountryTaxBenefitSystem()
time_spent_tbs = time.time() - start_time_tbs


openfisca_france_dir = pkg_resources.get_distribution('OpenFisca-France').location
yaml_tests_dir = os.path.join(openfisca_france_dir, 'tests', 'mes-aides.gouv.fr')


# Time openfisca-run-test runner
start_time_tests = time.time()
run_tests(tbs, yaml_tests_dir)
time_spent_tests = time.time() - start_time_tests


def compare_performance(baseline, test_result):
    delta = (test_result - baseline) * 100 / baseline

    if test_result > baseline * 1.2:
        logger.warning('The perfomance seems to have worsen by {} %.'.format(delta))
    elif test_result < baseline * 0.8:
        logger.info('The performance seems to have been improved by {} %.'.format(delta))
    else:
        logging.info('The performance seems steady ({} %).'.format(delta))


logger.info('Generate Tax Benefit System: --- {}s seconds ---'.format(time_spent_tbs))
compare_performance(BASELINE_TBS_LOAD_TIME, time_spent_tbs)

logger.info('Pass Mes-aides tests: --- {}s seconds ---'.format(time_spent_tests))
compare_performance(BASELINE_YAML_TESTS_TIME, time_spent_tests)



from openfisca_france.model.base import *

# PLAN :
# 1) Revenus des valeurs et capitaux mobiliers taxés au prélèvement libératoire
# 2) Revenus des valeurs et capitaux mobiliers ouvrant droit à abattement
# 3) Revenus des valeurs et capitaux mobiliers n'ouvrant pas droit à abattement
# 4) Autres revenus des valeurs et capitaux mobiliers
# 5) Revenus des valeurs et capitaux mobiliers utilisés par mes aides
# 6) Variables agrégées de revenus des valeurs et capitaux mobiliers


# Revenus des valeurs et capitaux mobiliers taxés au prélèvement libératoire

class f2da(Variable):
    cerfa_field = '2DA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Revenus des actions et parts soumis au prélèvement libératoire de 21 %'
    # start_date = date(2008, 1, 1)
    end = '2012-12-31'
    definition_period = YEAR


class f2dh(Variable):
    cerfa_field = '2DH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Produits d’assurance-vie et de capitalisation soumis au prélèvement libératoire de 7.5 %'
    definition_period = YEAR

    def formula_2013_01_01(foyer_fiscal, period):
        assurance_vie_pl_non_anonyme_plus8ans_depuis1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_plus8ans_depuis1990', period)
        assurance_vie_pl_non_anonyme_plus6ans_avant1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_plus6ans_avant1990', period)

        return assurance_vie_pl_non_anonyme_plus8ans_depuis1990 + assurance_vie_pl_non_anonyme_plus6ans_avant1990

    def formula_2018_01_01(foyer_fiscal, period):
        assurance_vie_pfu_ir_plus8ans_1990_19970926 = foyer_fiscal('assurance_vie_pfu_ir_plus8ans_1990_19970926', period)
        assurance_vie_pfu_ir_plus6ans_avant1990 = foyer_fiscal('assurance_vie_pfu_ir_plus6ans_avant1990', period)
        assurance_vie_pfu_ir_plus8ans_19970926_primes_avant_20170927 = foyer_fiscal('assurance_vie_pfu_ir_plus8ans_19970926_primes_avant_20170927', period)

        return (
            assurance_vie_pfu_ir_plus8ans_1990_19970926
            + assurance_vie_pfu_ir_plus6ans_avant1990
            + assurance_vie_pfu_ir_plus8ans_19970926_primes_avant_20170927
            )


class f2ee(Variable):
    cerfa_field = '2EE'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Autres produits de placement soumis aux prélèvements libératoires'
    definition_period = YEAR

    def formula_2013_01_01(foyer_fiscal, period):
        assurance_vie_pl_non_anonyme_moins4ans_depuis1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_moins4ans_depuis1990', period)
        assurance_vie_pl_non_anonyme_4_8_ans_depuis1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_4_8_ans_depuis1990', period)
        assurance_vie_pl_anonyme = foyer_fiscal('assurance_vie_pl_anonyme', period)
        produit_epargne_solidaire = foyer_fiscal('produit_epargne_solidaire', period)
        produit_etats_non_cooperatif = foyer_fiscal('produit_etats_non_cooperatif', period)

        return (
            assurance_vie_pl_non_anonyme_moins4ans_depuis1990
            + assurance_vie_pl_non_anonyme_4_8_ans_depuis1990
            + assurance_vie_pl_anonyme
            + produit_epargne_solidaire
            + produit_etats_non_cooperatif
            )

    def formula_2018_01_01(foyer_fiscal, period):
        '''
        Même si le formulaire 2042 de l'impôt 2019 au titre des revenus 2018 n'est toujours pas publié, on remplie cette case
        quand même, en essayant de garder un périmètre similaire. On fait ceci afin de ne pas modifier la structure des cases,
        ce qui pourrait avoir des impacts assez large dans le simulateur.
        Ce que l'on fait : on a toute une série de variables d'assurance-vie désagrégées, que l'on injecte en f2dh ou f2ee.
        En revanche, on neutralise les variables f2ch et f2ts
        '''
        assurance_vie_pfu_ir_moins4ans_1990_19970926 = foyer_fiscal('assurance_vie_pfu_ir_moins4ans_1990_19970926', period)
        assurance_vie_pfu_ir_4_8_ans_1990_19970926 = foyer_fiscal('assurance_vie_pfu_ir_4_8_ans_1990_19970926', period)
        assurance_vie_pfu_ir_4_8_ans_19970926_primes_avant_20170927 = foyer_fiscal('assurance_vie_pfu_ir_4_8_ans_19970926_primes_avant_20170927', period)
        assurance_vie_pfu_ir_moins4ans_19970926_primes_avant_20170927 = foyer_fiscal('assurance_vie_pfu_ir_moins4ans_19970926_primes_avant_20170927', period)
        produit_epargne_solidaire = foyer_fiscal('produit_epargne_solidaire', period)
        produit_etats_non_cooperatif = foyer_fiscal('produit_etats_non_cooperatif', period)

        return (
            assurance_vie_pfu_ir_moins4ans_1990_19970926
            + assurance_vie_pfu_ir_4_8_ans_1990_19970926
            + assurance_vie_pfu_ir_4_8_ans_19970926_primes_avant_20170927
            + assurance_vie_pfu_ir_moins4ans_19970926_primes_avant_20170927
            + produit_epargne_solidaire
            + produit_etats_non_cooperatif
            )


class f2xx(Variable):
    cerfa_field = '2XX'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Produits des bons et contrats de capitalisation et d'assurance-vie de moins de 8 ans; produits des versements effectués avant le 27.9.2017; soumis au prélèvement libératoire"
    # start_date = date(2108, 1, 1)
    definition_period = YEAR


# Revenus des valeurs et capitaux mobiliers ouvrant droit à abattement

class f2dc(Variable):
    cerfa_field = '2DC'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Revenus des actions et parts donnant droit à abattement'
    definition_period = YEAR


class f2fu(Variable):
    cerfa_field = '2FU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Revenus imposables des titres non côtés détenus dans le PEA et distributions perçues via votre entreprise donnant droit à abattement'
    definition_period = YEAR


class f2tq(Variable):
    cerfa_field = '2TQ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Intérêts imposables des obligations remboursables en actions détenues dans le PEA-PME'
    definition_period = YEAR


class f2ch(Variable):
    cerfa_field = '2CH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Produits des contrats d'assurance-vie et de capitalisation d'une durée d'au moins 6 ou 8 ans donnant droit à abattement"
    definition_period = YEAR


# Revenus des valeurs et capitaux mobiliers n'ouvrant pas droit à abattement

class f2ts(Variable):
    cerfa_field = '2TS'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Revenus de valeurs mobilières, produits des contrats d'assurance-vie d'une durée inférieure à 8 ans et distributions (n'ouvrant pas droit à abattement)"
    definition_period = YEAR


class f2go(Variable):
    cerfa_field = '2GO'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Autres revenus distribués et revenus des structures soumises hors de France à un régime fiscal privilégié (n'ouvrant pas droit à abattement)"
    definition_period = YEAR


class f2tr(Variable):
    cerfa_field = '2TR'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Produits de placements à revenu fixe, intérêts et autres revenus assimilés (n'ouvrant pas droit à abattement)"
    definition_period = YEAR


class f2yy(Variable):
    cerfa_field = '2YY'
    value_type = float
    entity = FoyerFiscal
    label = "Produits des bons ou contrats de capitalisation et d'assurance vie de moins de 8 ans pour les contrats souscrits après le 26 septembre 1997, dont le produits sont associés aux primes versées avant le 27 septembre 2017, et qui n'ont pas été soumis au prélèvement libératoire"
    definition_period = YEAR
    # start_date = date(2018, 1, 1)


# Autres revenus des valeurs et capitaux mobiliers

class f2fa(Variable):
    cerfa_field = '2FA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Intérêts et autres produits de placement à revenu fixe n'excédant pas 2000 euros, taxables sur option à 24%"
    # start_date = date(2013, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


class f2tt_2016(Variable):
    cerfa_field = '2TT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Intérêts des prêts participatifs'
    # start_date = date(2016, 1, 1)
    end = '2016-12-31'
    definition_period = YEAR


class f2tt(Variable):
    cerfa_field = '2TT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Intérêts des prêts participatifs nettes des pertes à imputer au titre de l'impôt sur le revenu"
    # start_date = date(2017, 1, 1)
    definition_period = YEAR


class f2tu_2016(Variable):
    cerfa_field = '2TU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Pertes en capital sur prêts participatifs en 2016'
    # start_date = date(2016, 1, 1)
    end = '2016-12-31'
    definition_period = YEAR


class f2tu(Variable):
    cerfa_field = '2TU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Pertes en capital sur prêts participatifs en 2016 à reporter sur l'année courante"
    # start_date = date(2017, 1, 1)
    definition_period = YEAR


class f2tv(Variable):
    cerfa_field = '2TV'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Pertes en capital sur prêts participatifs en 2017 à reporter sur l'année courante"
    # start_date = date(2017, 1, 1)
    definition_period = YEAR


class f2tw(Variable):
    cerfa_field = '2TW'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Pertes en capital sur prêts participatifs en 2017 à reporter sur l'année courante"
    # start_date = date(2018, 1, 1)
    definition_period = YEAR


class f2cg(Variable):
    cerfa_field = '2CG'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Revenus des lignes 2DC, 2CH, 2TS, 2TR déjà soumis au prélèvement sociaux sans CSG déductible'
    definition_period = YEAR


class f2bh(Variable):
    cerfa_field = '2BH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Revenus des lignes 2DC, 2CH, 2TS, 2TR déjà soumis au prélèvement sociaux avec CSG déductible'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f2df(Variable):
    cerfa_field = '2DF'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Autres revenus déjà soumis aux prélèvements sociaux avec CSG déductible'
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


class f2ca(Variable):
    cerfa_field = '2CA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Frais et charges déductibles'
    definition_period = YEAR


class f2ck(Variable):
    cerfa_field = '2CK'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Crédit d'impôt égal au prélèvement forfaitaire déjà versé"
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class f2ab(Variable):
    cerfa_field = '2AB'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Crédits d'impôt sur valeurs étrangères"
    definition_period = YEAR


class f2bg(Variable):
    cerfa_field = '2BG'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Crédits d'impôt 'directive épargne' et autres crédits d'impôt restituables"
    end = '2017-12-31'
    definition_period = YEAR


class f2aa(Variable):
    cerfa_field = '2AA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des années antérieures non encore déduits'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f2al(Variable):
    cerfa_field = '2AL'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des années antérieures non encore déduits'
    # start_date = date(2008, 1, 1)
    definition_period = YEAR


class f2am(Variable):
    cerfa_field = '2AM'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des années antérieures non encore déduits'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


class f2an(Variable):
    cerfa_field = '2AN'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des années antérieures non encore déduits'
    # start_date = date(2010, 1, 1)
    definition_period = YEAR


class f2aq(Variable):
    cerfa_field = '2AQ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des années antérieures non encore déduits'
    # start_date = date(2011, 1, 1)
    definition_period = YEAR


class f2ar(Variable):
    cerfa_field = '2AR'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des années antérieures non encore déduits'
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


class f2dm(Variable):
    cerfa_field = '2DM'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Impatriés: revenus de capitaux mobiliers perçus à l'étranger, abattement de 50 %"
    # start_date = date(2008, 1, 1)
    definition_period = YEAR


class f2gr(Variable):
    cerfa_field = '2GR'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Revenus distribués dans le PEA (pour le calcul du crédit d'impôt de 50 %)"
    # start_date = date(2005, 1, 1)
    end = '2009-12-31'
    definition_period = YEAR


# Revenus des valeurs et capitaux mobiliers utilisés par mes aides. TODO: à consolider

class livret_a(Variable):
    value_type = float
    entity = Individu
    label = 'Épargne sur Livret A'
    definition_period = MONTH
    is_period_size_independent = True
    set_input = set_input_dispatch_by_period


class epargne_revenus_non_imposables(Variable):
    ''' NB : cette variable est définie indépendemment des variables commençant par interets_plan_epargne_logement et interets_compte_epargne_logement '''
    value_type = float
    entity = Individu
    label = 'Épargne générant des revenus non imposables hors Livret A'
    definition_period = MONTH
    is_period_size_independent = True
    set_input = set_input_dispatch_by_period


class epargne_revenus_imposables(Variable):
    value_type = float
    entity = Individu
    label = 'Épargne générant des revenus imposables'
    definition_period = MONTH
    is_period_size_independent = True
    set_input = set_input_dispatch_by_period


# Variables agrégées de revenus des valeurs et capitaux mobiliers

class revenus_capitaux_prelevement_bareme(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenus du capital imposés au barème (montants bruts)'
    set_input = set_input_divide_by_period
    reference = 'http://bofip.impots.gouv.fr/bofip/3775-PGP'
    definition_period = MONTH

    def formula(foyer_fiscal, period, parameters):
        year = period.this_year
        f2dc = foyer_fiscal('f2dc', year)
        f2gr = foyer_fiscal('f2gr', year)
        f2ch = foyer_fiscal('f2ch', year)
        f2ts = foyer_fiscal('f2ts', year)
        f2go = foyer_fiscal('f2go', year)
        f2tr = foyer_fiscal('f2tr', year)
        f2fu = foyer_fiscal('f2fu', year)
        majoration_revenus_reputes_distribues = parameters(period).impot_revenu.calcul_revenus_imposables.rvcm.majoration_revenus_reputes_distribues

        return (f2dc + f2gr + f2ch + f2ts + f2go * majoration_revenus_reputes_distribues + f2tr + f2fu) / 12

    def formula_2013_01_01(foyer_fiscal, period, parameters):
        year = period.this_year
        f2dc = foyer_fiscal('f2dc', year)
        f2ch = foyer_fiscal('f2ch', year)
        f2ts = foyer_fiscal('f2ts', year)
        f2go = foyer_fiscal('f2go', year)
        f2tr = foyer_fiscal('f2tr', year)
        f2fu = foyer_fiscal('f2fu', year)
        majoration_revenus_reputes_distribues = parameters(period).impot_revenu.calcul_revenus_imposables.rvcm.majoration_revenus_reputes_distribues

        return (f2dc + f2ch + f2ts + f2go * majoration_revenus_reputes_distribues + f2tr + f2fu) / 12

    def formula_2016_01_01(foyer_fiscal, period, parameters):
        year = period.this_year
        f2dc = foyer_fiscal('f2dc', year)
        f2ch = foyer_fiscal('f2ch', year)
        f2ts = foyer_fiscal('f2ts', year)
        f2go = foyer_fiscal('f2go', year)
        f2tr = foyer_fiscal('f2tr', year)
        f2fu = foyer_fiscal('f2fu', year)
        f2tt_2016 = foyer_fiscal('f2tt_2016', year)
        majoration_revenus_reputes_distribues = parameters(period).impot_revenu.calcul_revenus_imposables.rvcm.majoration_revenus_reputes_distribues

        return (f2dc + f2ch + f2ts + f2go * majoration_revenus_reputes_distribues + f2tr + f2fu + f2tt_2016) / 12

    def formula_2017_01_01(foyer_fiscal, period, parameters):
        '''
        Note : à partir des revenus 2017, la case 2TT représente les intérêts des prêts participatifs nets des pertes à imputer,
        contrairement à avant où il s'agissait des intérêts avant déduction des pertes. Donc, à partir de 2017, on a ici,
        par contrainte de la structure des cases de la déclaration des revenus, le montant net au lieu du montant brut.
        '''
        year = period.this_year
        f2dc = foyer_fiscal('f2dc', year)
        f2ch = foyer_fiscal('f2ch', year)
        f2ts = foyer_fiscal('f2ts', year)
        f2go = foyer_fiscal('f2go', year)
        f2tr = foyer_fiscal('f2tr', year)
        f2fu = foyer_fiscal('f2fu', year)
        f2tt = foyer_fiscal('f2tt', year)
        majoration_revenus_reputes_distribues = parameters(period).impot_revenu.calcul_revenus_imposables.rvcm.majoration_revenus_reputes_distribues

        return (f2dc + f2ch + f2ts + f2go * majoration_revenus_reputes_distribues + f2tr + f2fu + f2tt) / 12

    def formula_2018_01_01(foyer_fiscal, period, parameters):
        year = period.this_year
        f2ch = foyer_fiscal('f2ch', year)
        f2yy = foyer_fiscal('f2yy', year)

        return (f2ch + f2yy) / 12


class revenus_capitaux_prelevement_liberatoire(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu du capital imposé au prélèvement libératoire (montants bruts)'
    set_input = set_input_divide_by_period
    reference = 'http://bofip.impots.gouv.fr/bofip/3817-PGP'
    definition_period = MONTH

    def formula_2002_01_01(foyer_fiscal, period, parameters):
        f2dh = foyer_fiscal('f2dh', period.this_year)
        f2ee = foyer_fiscal('f2ee', period.this_year)

        return (f2dh + f2ee) / 12

    def formula_2008_01_01(foyer_fiscal, period, parameters):
        f2da = foyer_fiscal('f2da', period.this_year)
        f2dh = foyer_fiscal('f2dh', period.this_year)
        f2ee = foyer_fiscal('f2ee', period.this_year)

        return (f2da + f2dh + f2ee) / 12

    def formula_2013_01_01(foyer_fiscal, period, parameters):
        f2dh = foyer_fiscal('f2dh', period.this_year)
        f2ee = foyer_fiscal('f2ee', period.this_year)
        f2fa = foyer_fiscal('f2fa', period.this_year)

        return (f2dh + f2ee + f2fa) / 12

    def formula_2018_01_01(foyer_fiscal, period, parameters):
        f2dh = foyer_fiscal('f2dh', period.this_year)
        f2ee = foyer_fiscal('f2ee', period.this_year)
        f2xx = foyer_fiscal('f2xx', period.this_year)

        return (f2dh + f2ee + f2xx) / 12


class revenus_capital(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus du capital'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        revenus_capitaux = (
            max_(0, individu.foyer_fiscal('revenus_capitaux_prelevement_bareme', period))
            + max_(0, individu.foyer_fiscal('revenus_capitaux_prelevement_liberatoire', period))
            + max_(0, individu.foyer_fiscal('revenus_capitaux_prelevement_forfaitaire_unique_ir', period))
            ) * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)

        return revenus_capitaux



from numpy import absolute as abs_, logical_and as and_

from openfisca_core.periods import Period

from openfisca_france.model.base import *


class ass_precondition_remplie(Variable):
    value_type = bool
    entity = Individu
    label = "Éligible à l'ASS"
    definition_period = MONTH
    reference = [
        'Article R5423-1 du Code du travail',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006072050&idArticle=LEGIARTI000018525084&dateTexte=20190618&categorieLien=cid#LEGIARTI000018525084'
        ]
    set_input = set_input_dispatch_by_period


class ass(Variable):
    value_type = float
    label = "Montant de l'ASS pour un individu"
    reference = [
        'https://www.service-public.fr/particuliers/vosdroits/F12484',
        'https://www.legifrance.gouv.fr/codes/section_lc/LEGITEXT000006072050/LEGISCTA000018496266/'
        ]
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        ass_base_ressources = individu.famille('ass_base_ressources', period)
        en_couple = individu.famille('en_couple', period)
        residence_mayotte = individu.menage('residence_mayotte', period)
        ass_params = parameters(period).chomage.allocations_assurance_chomage.ass

        elig = individu('ass_eligibilite_individu', period)

        montant_journalier = where(residence_mayotte, ass_params.montant_plein_mayotte, ass_params.montant_plein)
        montant_mensuel = 30 * montant_journalier
        plafond_mensuel = montant_journalier * (
            not_(en_couple) * ass_params.plaf_seul
            + en_couple * ass_params.plaf_coup
            )
        revenus = ass_base_ressources / 12

        ass = min_(montant_mensuel, plafond_mensuel - revenus)
        ass = max_(ass, 0)
        ass = ass * elig
        # pas d'ASS si montant mensuel < montant journalier de base
        ass = ass * not_(ass < ass_params.montant_plein)

        return ass


class ass_base_ressources(Variable):
    value_type = float
    label = "Base de ressources de l'ASS"
    entity = Famille
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        ass_base_ressources_demandeur = famille.demandeur('ass_base_ressources_individu', period)
        ass_base_ressources_conjoint = famille.conjoint('ass_base_ressources_conjoint', period)

        result = ass_base_ressources_demandeur + ass_base_ressources_conjoint
        return result


class ass_base_ressources_individu(Variable):
    value_type = float
    label = "Base de ressources individuelle de l'ASS"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        # Articles R5423-1 à 6 du code du travail
        'https://www.legifrance.gouv.fr/affichCode.do?idSectionTA=LEGISCTA000018525086&cidTexte=LEGITEXT000006072050&dateTexte=20181227'
        ]

    def formula(individu, period, parameters):
        # Rolling year
        previous_year = Period(('year', period.start, 1)).offset(-1)
        # N-1
        last_year = period.last_year

        salaire_imposable = calculateWithAbatement(individu, parameters, period, 'salaire_imposable')
        revenus_stage_formation_pro = calculateWithAbatement(individu, parameters, period, 'revenus_stage_formation_pro')
        aah = calculateWithAbatement(individu, parameters, period, 'aah')
        retraite_nette = calculateWithAbatement(individu, parameters, period, 'retraite_nette')
        pensions_alimentaires_percues = calculateWithAbatement(individu, parameters, period, 'pensions_alimentaires_percues')
        indemnites_stage = calculateWithAbatement(individu, parameters, period, 'indemnites_stage')

        pensions_invalidite = individu('pensions_invalidite', previous_year, options=[ADD])
        revenus_locatifs = individu('revenus_locatifs', previous_year, options=[ADD])
        revenus_capital = individu('revenus_capital', period)

        def revenus_tns():
            revenus_auto_entrepreneur = individu('rpns_auto_entrepreneur_benefice', previous_year, options=[ADD])

            # Les revenus TNS hors AE sont estimés en se basant sur le revenu N-1
            rpns_micro_entreprise_benefice = individu('rpns_micro_entreprise_benefice', last_year)
            rpns_benefice_exploitant_agricole = individu('rpns_benefice_exploitant_agricole', last_year)
            rpns_autres_revenus = individu('rpns_autres_revenus', last_year)

            return revenus_auto_entrepreneur + rpns_micro_entreprise_benefice + rpns_benefice_exploitant_agricole + rpns_autres_revenus

        pensions_alimentaires_versees_individu = individu('pensions_alimentaires_versees_individu', previous_year, options=[ADD])

        return (
            salaire_imposable
            + retraite_nette
            + pensions_invalidite
            + pensions_alimentaires_percues
            - abs_(pensions_alimentaires_versees_individu)
            + aah
            + indemnites_stage
            + revenus_stage_formation_pro
            + revenus_tns()
            + revenus_locatifs
            + revenus_capital
            )


class ass_base_ressources_conjoint(Variable):
    value_type = float
    label = "Base de ressources individuelle pour le conjoint du demandeur de l'ASS"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        # Rolling year
        previous_year = Period(('year', period.start, 1)).offset(-1)

        last_month = Period(('month', period.start, 1)).offset(-1)

        ass_base_ressources_individu = individu('ass_base_ressources_individu', period)
        chomage_net_interrompue = individu('chomage_net', last_month) == 0
        chomage_net = individu('chomage_net', previous_year, options=[ADD]) * (1 - chomage_net_interrompue)
        indemnites_journalieres = calculateWithAbatement(individu, parameters, period, 'indemnites_journalieres')

        return (
            ass_base_ressources_individu
            + chomage_net
            + indemnites_journalieres
            )


def calculateWithAbatement(individu, parameters, period, ressourceName):
    last_month = Period(('month', period.start, 1)).offset(-1)
    has_ressources_substitution = (
        individu('chomage_net', last_month)
        + individu('indemnites_journalieres', last_month)
        + individu('retraite_nette', last_month)
        ) > 0
    # Rolling year
    previous_year = Period(('year', period.start, 1)).offset(-1)

    ressource_year = individu(ressourceName, previous_year, options=[ADD])
    ressource_last_month = individu(ressourceName, last_month)

    ressource_interrompue = (ressource_year > 0) * (ressource_last_month == 0)

    # Les ressources interrompues sont abattues différement si elles sont substituées ou non.
    # http://www.legifrance.gouv.fr/affichCodeArticle.do?idArticle=LEGIARTI000020398006&cidTexte=LEGITEXT000006072050

    tx_abat_partiel = parameters(period).chomage.allocations_assurance_chomage.ass.abat_rev_subst_conj
    tx_abat_total = parameters(period).chomage.allocations_assurance_chomage.ass.abat_rev_non_subst_conj

    tx_abat_applique = where(has_ressources_substitution, tx_abat_partiel, tx_abat_total)

    return where(ressource_interrompue, (1 - tx_abat_applique) * ressource_year, ressource_year)


class ass_eligibilite_cumul_individu(Variable):
    value_type = bool
    label = "Eligibilité au cumul de l'ASS avec un revenu d'activité"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'Article R5425-2 du code du travail',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006072050&idArticle=LEGIARTI000018496568&dateTexte=',
        'https://www.legifrance.gouv.fr/eli/decret/2017/5/5/ETSD1708117D/jo/article_2'
        ]

    def formula_2017_09_01(individu, period):
        douze_mois_precedents = [period.offset(offset) for offset in range(-12, 0 + 1)]
        nb_mois_cumul = 0
        nb_mois_consecutif_sans_activite = 0

        for mois in douze_mois_precedents:
            presence_ressources_activite = individu('salaire_imposable', mois) > 0
            absence_ressources_activite = not_(presence_ressources_activite)
            ass_precondition_remplie = individu('ass_precondition_remplie', mois)
            chomeur = individu('activite', mois) == TypesActivite.chomeur
            absence_aah = not_(individu('aah', mois) > 0)

            # reinitialisation du nombre de mois de cumul après 3 mois consécutif sans activité
            nb_mois_cumul = nb_mois_cumul * (nb_mois_consecutif_sans_activite < 3)
            nb_mois_consecutif_sans_activite = where(absence_ressources_activite * chomeur, nb_mois_consecutif_sans_activite + 1, 0)

            nb_mois_cumul = (
                nb_mois_cumul
                + 1
                * presence_ressources_activite
                * not_(chomeur)
                * ass_precondition_remplie
                * absence_aah
                )

        # si 3 mois de cumul ou moins en comptant le mois courant, droit au cumul au moins courant
        return nb_mois_cumul <= 3


class ass_eligibilite_individu(Variable):
    value_type = bool
    label = "Éligibilité individuelle à l'ASS"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'Article L5423-1 du code du travail',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006072050&idArticle=LEGIARTI000006903847&dateTexte=&categorieLien=cid',
        ]

    def formula(individu, period, parameters):
        age_max = parameters(period).chomage.allocations_assurance_chomage.ass.age_max
        sous_age_limite = individu('age_en_mois', period) <= age_max

        demandeur_emploi_non_indemnise = and_(individu('activite', period) == TypesActivite.chomeur, individu('chomage_net', period) == 0)

        # Indique que l'individu a travaillé 5 ans au cours des 10 dernieres années.
        ass_precondition_remplie = individu('ass_precondition_remplie', period)

        return demandeur_emploi_non_indemnise * ass_precondition_remplie * sous_age_limite

    def formula_2017_01_01(individu, period, parameters):
        age_max = parameters(period).chomage.allocations_assurance_chomage.ass.age_max
        sous_age_limite = individu('age_en_mois', period) <= age_max

        aah_eligible = individu('aah', period) > 0

        eligible_cumul_ass = individu('ass_eligibilite_cumul_individu', period)

        demandeur_emploi_non_indemnise_et_cumul_accepte = (
            (individu('chomage_net', period) == 0)
            * ((individu('activite', period) == TypesActivite.chomeur) + eligible_cumul_ass)
            )

        # Indique que l'individu a travaillé 5 ans au cours des 10 dernieres années.
        ass_precondition_remplie = individu('ass_precondition_remplie', period)

        return not_(aah_eligible) * demandeur_emploi_non_indemnise_et_cumul_accepte * ass_precondition_remplie * sous_age_limite



from openfisca_france.model.base import *


class cheque_energie_unites_consommation(Variable):
    entity = Menage
    value_type = float
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=1EA40CA7787AF90A95D1E1B3155D9028.tplgfr29s_1?idArticle=LEGIARTI000032497834&cidTexte=LEGITEXT000023983208&dateTexte=20160511'
    label = 'Unités de consommation du ménage pour le calcul du chèque Énergie'
    definition_period = YEAR

    def formula_2017(menage, period, parameters):
        uc = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.cheque_energie.unites_consommation
        nb_personnes = menage.nb_persons()
        gardes_alternees = menage.sum(menage.members('garde_alternee', period.first_month))

        nb_personnes_ajuste = nb_personnes - 0.5 * gardes_alternees
        return (
            uc.premiere_personne
            + uc.deuxieme_personne * (nb_personnes_ajuste > 1) * (min_(nb_personnes_ajuste, 2) - 1)
            + uc.autres_personnes * (nb_personnes_ajuste > 2) * (nb_personnes_ajuste - 2)
            )


class cheque_energie_eligibilite_logement(Variable):
    entity = Menage
    value_type = bool
    reference = [
        "Article L124-1 du Code de l'énergie",
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=5AB50D02153C9CB753729850314A2E17.tplgfr29s_1?idArticle=LEGIARTI000031057544&cidTexte=LEGITEXT000023983208&dateTexte=20180314',
        'Article LO6314-3 du Code général des collectivités territoriales',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=6A3717E70623B148432581CC8F585C5F.tplgfr31s_1?idArticle=LEGIARTI000006394061&cidTexte=LEGITEXT000006070633&dateTexte=20180316',
        ]
    label = 'Éligibilité du logement occupé au chèque énergie'
    definition_period = YEAR
    set_input = set_input_dispatch_by_period

    def formula_2017(menage, period, parameters):
        statut_occupation_logement = menage('statut_occupation_logement', period.first_month)
        residence_saint_martin = menage('residence_saint_martin', period.first_month)

        return (
            not_(residence_saint_martin) * (
                + (statut_occupation_logement == TypesStatutOccupationLogement.primo_accedant)
                + (statut_occupation_logement == TypesStatutOccupationLogement.proprietaire)
                + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_hlm)
                + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_vide)
                + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_meuble)
                )
            )


class cheque_energie_montant(Variable):
    entity = Menage
    value_type = float
    reference = [
        'https://www.legifrance.gouv.fr/eli/decret/2016/5/6/DEVR1604032D/jo/article_1',
        'https://www.legifrance.gouv.fr/eli/arrete/2018/12/26/TRER1832961A/jo/texte',
        ]
    label = 'Montant du chèque énergie'
    definition_period = YEAR

    def formula_2017(menage, period, parameters):
        baremes = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.cheque_energie.baremes

        uc_menage = menage('cheque_energie_unites_consommation', period)
        rfr = menage.sum(menage.members.foyer_fiscal('rfr', period.n_2), role = FoyerFiscal.DECLARANT_PRINCIPAL)

        ressources_par_uc = rfr / uc_menage

        return (
            (uc_menage <= 1) * baremes.une_uc.calc(ressources_par_uc)
            + ((uc_menage > 1) * (uc_menage < 2)) * baremes.une_uc_a_deux_uc.calc(ressources_par_uc)
            + (uc_menage >= 2) * baremes.plus_de_deux_ucs.calc(ressources_par_uc)
            )


class cheque_energie(Variable):
    entity = Menage
    value_type = float
    reference = 'https://chequeenergie.gouv.fr'
    label = 'Montant auquel le ménage peut prétendre au titre du chèque energie'
    definition_period = YEAR
    set_input = set_input_divide_by_period

    def formula_2017(menage, period):
        eligible = menage('cheque_energie_eligibilite_logement', period)
        declarant = menage.sum(menage.members('age', period.first_month) * 0 + 1, role = FoyerFiscal.DECLARANT) > 0  # une colocation de personnes à la charge de leurs parents n'est pas éligible aux chèques énergie, par exemple
        montant = menage('cheque_energie_montant', period.this_year)
        return declarant * eligible * montant


class cheque_energie_exceptionnel_montant(Variable):
    entity = Menage
    value_type = float
    reference = ['https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044387624',
        'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000046720566']
    label = 'Montant du chèque énergie exceptionnel'
    definition_period = MONTH
    end = '2022-12-31'

    def formula_2021_12_01(menage, period, parameters):
        bareme = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.cheque_energie.aide_exceptionnelle

        uc_menage = menage('cheque_energie_unites_consommation', period.this_year)
        rfr = menage.sum(menage.members.foyer_fiscal('rfr', period.n_2), role = FoyerFiscal.DECLARANT_PRINCIPAL)

        ressources_par_uc = rfr / uc_menage

        return bareme.calc(ressources_par_uc)


class aide_exceptionnelle_cheque_energie(Variable):
    entity = Menage
    value_type = float
    label = 'Aide exceptionnelle sous forme de cheque energie a partir de 2021'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2021(menage, period, parameters):
        eligible = menage('cheque_energie_eligibilite_logement', period)
        declarant = menage.sum(menage.members('age', period.first_month) * 0 + 1, role = FoyerFiscal.DECLARANT) > 0  # une colocation de personnes à la charge de leurs parents n'est pas éligible aux chèques énergie, par exemple
        montant = menage('cheque_energie_exceptionnel_montant', period.first_month)
        return declarant * eligible * montant



import os

from ..model.base import *


dir_path = os.path.dirname(__file__)


# Réforme de l'amendement Ayrault-Muet

def ayrault_muet_modify_parameters(parameters):
    # TODO: inflater les paramètres de la décote le barème de l'IR
    inflator = 1
    for inflation in [2.8, 0.1, 1.5, 2.1, 2, 0.9, 0.5, 0.1]:
        inflator = inflator * (1 + inflation / 100)
    del inflation

    elig1 = Parameter('elig1', {'values': {'2015-01-01': {'value': round(16251 * inflator)}, '2016-01-01': {'value': None}}})
    elig2 = Parameter('elig2', {'values': {'2015-01-01': {'value': round(32498 * inflator)}, '2016-01-01': {'value': None}}})
    elig3 = Parameter('elig3', {'values': {'2015-01-01': {'value': round(4490 * inflator)}, '2016-01-01': {'value': None}}})
    parameters.impot_revenu.credits_impots.ppe.add_child('elig1', elig1)
    parameters.impot_revenu.credits_impots.ppe.add_child('elig2', elig2)
    parameters.impot_revenu.credits_impots.ppe.add_child('elig3', elig3)
    return parameters


class variator(Variable):
    value_type = float
    default_value = 1
    entity = FoyerFiscal
    label = 'Multiplicateur du seuil de régularisation'
    definition_period = YEAR


class reduction_csg(Variable):
    value_type = float
    entity = Individu
    label = 'Réduction dégressive de CSG'
    definition_period = YEAR

    def formula_2015_01_01(individu, period, parameters):
        smic_proratise = individu('smic_proratise', period, options = [ADD])
        assiette_csg_abattue = individu('assiette_csg_abattue', period, options = [ADD])

        seuil = 1.34
        coefficient_correctif = .9
        taux_csg = (
            parameters(period).csg.activite.imposable.taux
            + parameters(period).csg.activite.deductible.taux
            )
        tx_max = coefficient_correctif * taux_csg
        ratio_smic_salaire = smic_proratise / (assiette_csg_abattue + 1e-16)
        # règle d'arrondi: 4 décimales au dix-millième le plus proche
        taux_allegement_csg = tx_max * min_(1, max_(seuil - 1 / ratio_smic_salaire, 0) / (seuil - 1))
        # Montant de l'allegment
        return taux_allegement_csg * assiette_csg_abattue


class reduction_csg_foyer_fiscal(Variable):
    entity = FoyerFiscal
    label = 'Réduction dégressive de CSG des memebres du foyer fiscal'
    value_type = float
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        reduction_csg = foyer_fiscal('reduction_csg', period)
        return foyer_fiscal.sum(reduction_csg)


class reduction_csg_nette(Variable):
    value_type = float
    entity = Individu
    label = 'Réduction dégressive de CSG'
    definition_period = YEAR

    def formula_2015_01_01(individu, period):
        reduction_csg = individu('reduction_csg', period)
        ppe_elig_bis = individu.foyer_fiscal('ppe_elig_bis', period)
        return reduction_csg * ppe_elig_bis


class ppe_elig_bis(Variable):
    value_type = bool
    entity = FoyerFiscal
    label = 'ppe_elig_bis'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        PPE: eligibilité à la ppe, condition sur le revenu fiscal de référence
        'foy'
        '''
        rfr = foyer_fiscal('rfr', period)
        ppe_coef = foyer_fiscal('ppe_coef', period)
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)
        veuf = foyer_fiscal('veuf', period)
        celibataire_ou_divorce = foyer_fiscal('celibataire_ou_divorce', period)
        nbptr = foyer_fiscal('nbptr', period)
        variator = foyer_fiscal('variator', period)
        ppe = parameters(period).impot_revenu.credits_impots.ppe
        seuil = (veuf | celibataire_ou_divorce) * (ppe.seuils_rfr_eligibilite.personne_seule + 2 * max_(nbptr - 1, 0) * ppe.seuils_rfr_eligibilite.increment_par_demi_part)\
            + maries_ou_pacses * (ppe.seuils_rfr_eligibilite.couple_marie_pacse + 2 * max_(nbptr - 2, 0) * ppe.seuils_rfr_eligibilite.increment_par_demi_part)
        return (rfr * ppe_coef) <= (seuil * variator)


class regularisation_reduction_csg(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Régularisation complète réduction dégressive de CSG'
    definition_period = YEAR

    def formula_2015_01_01(foyer_fiscal, period, parameters):
        reduction_csg = foyer_fiscal('reduction_csg_foyer_fiscal', period)
        ppe_elig_bis = foyer_fiscal('ppe_elig_bis', period)
        return not_(ppe_elig_bis) * (reduction_csg > 1)


class ayrault_muet(Reform):
    name = 'Amendement Ayrault-Muet au PLF2016'
    key = 'ayrault_muet'

    def apply(self):
        for variable in [
                reduction_csg,
                regularisation_reduction_csg,
                reduction_csg_foyer_fiscal,
                reduction_csg_nette,
                ppe_elig_bis,
                variator,
                ]:
            self.update_variable(variable)
        self.modify_parameters(modifier_function = ayrault_muet_modify_parameters)



import math

from openfisca_france.model.base import *
from openfisca_france.model.prelevements_obligatoires.prelevements_sociaux.cotisations_sociales.base import apply_bareme_for_relevant_type_sal


class ati_atiacl(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation ATI et ATIACL (contributions pour le financement de l'allocation temporaires d'invalidité)"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # patronale, non-contributive

    def formula(individu, period, parameters):
        remuneration_principale = individu('remuneration_principale', period)
        categorie_salarie = individu('categorie_salarie', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        _P = parameters(period)

        # ATI : pour les fonctionnaires d'Etat, hors militaires
        cotisation_etat_hors_militaires = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = _P.cotsoc.cotisations_employeur,
            bareme_name = 'ati',
            base = remuneration_principale,
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )
        # ATIACL : pour les fonctionnaires territoriaux et hospitaliers
        cotisation_collectivites_locales = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = _P.cotsoc.cotisations_employeur,
            bareme_name = 'atiacl',
            base = remuneration_principale,
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )
        return cotisation_etat_hors_militaires + cotisation_collectivites_locales


# sft dans assiette csg et RAFP et Cotisation exceptionnelle de solidarité et taxe sur les salaires
# primes dont indemnites de residences idem sft
# avantages en nature contrib exceptionnelle de solidarite, RAFP, CSG, CRDS.


class contribution_exceptionnelle_solidarite(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation exceptionnelle au fonds de solidarité (salarié)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2017-12-31'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006072050&idArticle=LEGIARTI000006903878&dateTexte=&categorieLien=cid'

    def formula(individu, period, parameters):
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        hsup = individu('hsup', period)
        categorie_salarie = individu('categorie_salarie', period)
        indemnite_residence = individu('indemnite_residence', period)
        primes_fonction_publique = individu('primes_fonction_publique', period)
        rafp_salarie = individu('rafp_salarie', period)
        pension_salarie = individu('pension_salarie', period)
        cotisations_salariales_contributives = individu('cotisations_salariales_contributives', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        salaire_de_base = individu('salaire_de_base', period)
        supplement_familial_traitement = individu('supplement_familial_traitement', period)
        # Assujettis
        parameters = parameters(period)
        seuil_assujettissement_fds = compute_seuil_fds(parameters.prelevements_sociaux.cotisations_secteur_public.fds.salarie)
        concernes = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            + (categorie_salarie == TypesCategorieSalarie.public_non_titulaire)
            )
        remuneration_brute = (
            traitement_indiciaire_brut
            + salaire_de_base
            + indemnite_residence
            - hsup
            )
        assujettis = concernes * (remuneration_brute > seuil_assujettissement_fds)
        # Pour le calcul de l'assiette, on déduit de la rémunaration brute
        #  - toutes les cotisations de sécurité sociale obligatoires
        #  - les prélèvements pour pension
        #  - et, le cas échéant, les prélèvements au profit des régimes de retraite complémentaire obligatoires.
        # Soit:
        #  - pour les titutlaires, les pensions
        #  - les non titulaires, les cotisations sociales contributives (car pas de cotisations non contributives pour les non titulaires de la fonction public)
        deduction = assujettis * (
            + rafp_salarie
            + pension_salarie
            + (categorie_salarie == TypesCategorieSalarie.public_non_titulaire) * cotisations_salariales_contributives
            )
        # Ces déductions sont négatives
        cotisation = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = parameters.cotsoc.cotisations_salarie,
            bareme_name = 'excep_solidarite',
            base = assujettis * min_(
                remuneration_brute + supplement_familial_traitement + primes_fonction_publique + deduction,
                parameters.prelevements_sociaux.cotisations_secteur_public.fds.salarie.plafond_base_solidarite,
                ),
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )
        return cotisation


class indemnite_compensatrice_csg(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnité compensatrice créée en 2018 pour compenser les effets de la hausse de la CSG - Calcul pour les fonctionnnaires recrutés à partir de 2018'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2018_01_01(individu, period, parameters):
        remuneration_principale = individu('remuneration_principale', period)
        categorie_salarie = individu('categorie_salarie', period)
        eligible = ((categorie_salarie == TypesCategorieSalarie.public_titulaire_etat) + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire) + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale) + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)) > 0
        indem = remuneration_principale * 0.0076

        return indem * eligible


class fonds_emploi_hospitalier(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation au fonds pour l'emploi hospitalier (FEH) (cotisation employeur)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        remuneration_principale = individu('remuneration_principale', period)
        categorie_salarie = individu('categorie_salarie', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        _P = parameters(period)

        # Que pour fonctionnaires hospitaliers
        cotisation = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = _P.cotsoc.cotisations_employeur,
            bareme_name = 'feh',
            base = remuneration_principale,
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )
        return cotisation


class ircantec_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Ircantec salarié'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        categorie_salarie = individu('categorie_salarie', period)
        _P = parameters(period)

        ircantec = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = _P.cotsoc.cotisations_salarie,
            bareme_name = 'ircantec',
            base = assiette_cotisations_sociales,
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )

        return ircantec * (categorie_salarie == TypesCategorieSalarie.public_non_titulaire)


class ircantec_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Ircantec employeur'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        categorie_salarie = individu('categorie_salarie', period)
        _P = parameters(period)

        ircantec = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = _P.cotsoc.cotisations_employeur,
            bareme_name = 'ircantec',
            base = assiette_cotisations_sociales,
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )

        return ircantec * (categorie_salarie == TypesCategorieSalarie.public_non_titulaire)


class pension_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation au régime de base de retraite de la fonction publique - part salariale (retenue pour pension)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        nouvelle_bonification_indiciaire = individu('nouvelle_bonification_indiciaire', period)
        categorie_salarie = individu('categorie_salarie', period)
        _P = parameters(period)
        sal = _P.cotsoc.cotisations_salarie

        terr_or_hosp = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale) | (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            )
        etat_militaire = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            )

        montant = (
            etat_militaire
            * sal['public_titulaire_etat']['pension'].calc(traitement_indiciaire_brut + nouvelle_bonification_indiciaire)
            + terr_or_hosp * sal['public_titulaire_territoriale']['cnracl_s_ti'].calc(traitement_indiciaire_brut)
            + terr_or_hosp * sal['public_titulaire_territoriale']['cnracl_s_nbi'].calc(nouvelle_bonification_indiciaire)
            )

        return - montant


class pension_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation au régime de base de retraite de la fonction publique - part employeur'
    reference = 'http://www.ac-besancon.fr/spip.php?article2662'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        remuneration_principale = individu('remuneration_principale', period)
        categorie_salarie = individu('categorie_salarie', period)

        terr_or_hosp = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale) | (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            )
        etat = (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
        militaire = (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
        _P = parameters(period)
        pat = _P.cotsoc.cotisations_employeur

        montant = (
            etat * pat['public_titulaire_etat']['pension'].calc(remuneration_principale)
            + militaire * pat['public_titulaire_militaire']['pension'].calc(remuneration_principale)
            + terr_or_hosp * pat['public_titulaire_territoriale']['cnracl'].calc(remuneration_principale)
            )

        return - montant


class rafp_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Part salariale de la retraite additionelle de la fonction publique'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2005_01_01(individu, period, parameters):
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        categorie_salarie = individu('categorie_salarie', period)
        primes_fonction_publique = individu('primes_fonction_publique', period)
        supplement_familial_traitement = individu('supplement_familial_traitement', period)
        indemnite_residence = individu('indemnite_residence', period)
        gipa = individu('gipa', period)
        avantage_en_nature = individu('avantage_en_nature', period)

        eligible = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            )

        parametres_rafp = parameters(period).prelevements_sociaux.cotisations_secteur_public.rafp
        taux_plafond_tib = parametres_rafp.rafp_plaf_assiette
        _P = parameters(period)
        bareme_rafp_salarie = _P.cotsoc.cotisations_salarie.public_titulaire_etat['rafp']

        base_imposable = primes_fonction_publique + supplement_familial_traitement + indemnite_residence + avantage_en_nature
        assiette = (min_(base_imposable, taux_plafond_tib * traitement_indiciaire_brut) + gipa) * eligible
        # Même régime pour les fonctions publiques d'Etat et des collectivité locales
        rafp_salarie = eligible * bareme_rafp_salarie.calc(assiette)
        return - rafp_salarie


class rafp_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Part patronale de la retraite additionnelle de la fonction publique'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2005_01_01(individu, period, parameters):
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        categorie_salarie = individu('categorie_salarie', period)
        primes_fonction_publique = individu('primes_fonction_publique', period)
        supplement_familial_traitement = individu('supplement_familial_traitement', period)
        indemnite_residence = individu('indemnite_residence', period)
        indemnite_compensatrice_csg = individu('indemnite_compensatrice_csg', period)
        gipa = individu('gipa', period)
        avantage_en_nature = individu('avantage_en_nature', period)

        eligible = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            )

        parametres_rafp = parameters(period).prelevements_sociaux.cotisations_secteur_public.rafp
        taux_plafond_tib = parametres_rafp.rafp_plaf_assiette
        _P = parameters(period)
        bareme_rafp_employeur = _P.cotsoc.cotisations_employeur.public_titulaire_etat['rafp']

        base_imposable = primes_fonction_publique + supplement_familial_traitement + indemnite_residence + indemnite_compensatrice_csg + avantage_en_nature
        assiette = (min_(base_imposable, taux_plafond_tib * traitement_indiciaire_brut) + gipa) * eligible
        # Même régime pour les fonctions publiques d'Etat et des collectivité locales
        rafp_employeur = eligible * bareme_rafp_employeur.calc(assiette)
        return - rafp_employeur


def compute_seuil_fds(fds):
    '''
    Calcule le seuil mensuel d'assujetissement à la contribution au fond de solidarité
    '''
    pt_ind_mensuel = fds.pt_ind / 12
    seuil_mensuel = math.floor((pt_ind_mensuel * fds.ind_maj_ref))  # TODO improve
    return seuil_mensuel



'''Teste les cotisations par catégorie de salarié produites à l'issue de l'étape preprocessing.'''

from collections import OrderedDict

from openfisca_core.parameters.parameter_scale import ParameterScale
from openfisca_france.model.prelevements_obligatoires.prelevements_sociaux.cotisations_sociales.base import (
    cotisations_employeur_by_categorie_salarie,
    cotisations_salarie_by_categorie_salarie,
    )


from .cache import tax_benefit_system


cotisations_salarie_by_name = {
    'agff': {
        'start_non_null_date': '2001-04-01',
        'final_null_date': '2019-01-01',
        },
    'agirc_arrco': {
        'start_non_null_date': '2019-01-01',
        },
    'agirc': {
        'start_non_null_date': '1948-01-01',
        'final_null_date': '2019-01-01',
        },
    'apec': {
        'start_non_null_date': '1971-01-01',
        },
    'arrco': {
        'start_non_null_date': '1962-01-01',
        'final_null_date': '2019-01-01',
        },
    'asf': {
        'start_non_null_date': '1984-01-01',
        },
    'ceg': {
        'start_non_null_date': '2019-01-01',
        },
    'cet': {
        'start_non_null_date': '1997-01-01',
        'final_null_date': '2019-01-01',
        },
    'cet2019': {
        'start_non_null_date': '2019-01-01',
        },
    'chomage': {
        'start_non_null_date': '1959-01-01',
        'final_null_date': '2018-10-01',
        },
    'cnracl_s_ti': {
        'start_non_null_date': '1947-09-19',
        },
    'cnracl_s_nbi': {
        'start_non_null_date': '1947-09-19',
        },
    'excep_solidarite': {
        'start_non_null_date': '1982-11-01',
        'final_null_date': '2018-01-01',
        },
    'ircantec': {
        'start_non_null_date': '1971-01-01',
        },
    'maladie_alsace_moselle': {
        'start_non_null_date': '1989-09-01',
        },
    'maladie': {
        'start_non_null_date': '1967-10-01',
        'final_null_date': '2018-01-01',
        },
    'pension': {
        'start_non_null_date': '1925-01-01',
        },
    'rafp': {
        'start_non_null_date': '2005-01-01'
        },
    'vieillesse_deplafonnee': {
        'start_non_null_date': '2004-07-01'
        },
    'vieillesse_plafonnee': {
        'start_non_null_date': '1993-07-01',
        },
    }


def test_preprocessing():
    '''Tests the result of parameters preprocessing.'''
    parameters = tax_benefit_system.parameters
    assert set(parameters.cotsoc.cotisations_employeur.children.keys()) == set([
        'prive_cadre',
        'prive_non_cadre',
        'public_non_titulaire',
        'public_titulaire_etat',
        'public_titulaire_hospitaliere',
        'public_titulaire_militaire',
        'public_titulaire_territoriale',
        ]), 'Les barèmes de cotisations employeur de certaines catégories de salariés sont manquants'

    assert set(parameters.cotsoc.cotisations_salarie.children.keys()) == set([
        'prive_cadre',
        'prive_non_cadre',
        'public_non_titulaire',
        'public_titulaire_etat',
        'public_titulaire_hospitaliere',
        # 'public_titulaire_militaire',  FIXME Il y en a sûrement mais pas actuellement
        'public_titulaire_territoriale',
        ]), 'Les barèmes de cotisations salarié de certaines catégories instant_sde salariés sont manquants'

    categorie_salaries = [
        'prive_cadre',
        'prive_non_cadre',
        'public_non_titulaire',
        'public_titulaire_etat',
        'public_titulaire_hospitaliere',
        'public_titulaire_territoriale',
        ]

    for categorie_salarie in categorie_salaries:
        test = parameters.cotsoc.cotisations_salarie.children[categorie_salarie].children.keys()
        target = cotisations_salarie_by_categorie_salarie[categorie_salarie]
        assert set(test) == set(target), 'Les barèmes de cotisations salarié {} ne sont pas les bons'.format(
            categorie_salarie)

        test = parameters.cotsoc.cotisations_employeur.children[categorie_salarie].children.keys()
        target = cotisations_employeur_by_categorie_salarie[categorie_salarie]
        assert set(test) == set(target), 'Les barèmes de cotisations employeur {} ne sont pas les bons'.format(
            categorie_salarie)

    for categorie_salarie in categorie_salaries:
        cotisations_salaries = set(cotisations_salarie_by_categorie_salarie[categorie_salarie])
        for cotisation_salarie in sorted(cotisations_salaries):
            bareme = parameters.cotsoc.cotisations_salarie.children[categorie_salarie].children[cotisation_salarie]

            assert isinstance(bareme, ParameterScale), f"{cotisation_salarie} entry's class is {type(bareme)} which is not a ParameterScale"

            final_null_date = cotisations_salarie_by_name[cotisation_salarie].get('final_null_date')
            if final_null_date:
                thresholds = [
                    dict(
                        (parameter_at_instant.instant_str, parameter_at_instant.value)
                        for parameter_at_instant in bracket.threshold.values_list
                        )
                    for bracket in bareme.brackets
                    ]
                final_thresholds_by_instant_str = OrderedDict(sorted(threshold.items(), reverse = True)[0] for threshold in thresholds)
                assert all([final_threshold is None for final_threshold in final_thresholds_by_instant_str.values()]), "Barème salarié {} ne s'éteint pas (il devrait en {})".format(
                    cotisation_salarie,
                    final_null_date,
                    )
                assert max(final_thresholds_by_instant_str.keys()) == final_null_date, "Barème salarié {} ne s'éteint pas en {}".format(
                    cotisation_salarie,
                    final_null_date,
                    )

            start_non_null_date = cotisations_salarie_by_name[cotisation_salarie].get('start_non_null_date')
            if start_non_null_date:
                thresholds = [
                    dict(
                        (parameter_at_instant.instant_str, parameter_at_instant.value)
                        for parameter_at_instant in bracket.threshold.values_list
                        )
                    for bracket in bareme.brackets
                    ]
                start_thresholds_by_instant_str = OrderedDict(sorted(threshold.items())[0] for threshold in thresholds)
                assert all([start_threshold is not None for start_threshold in start_thresholds_by_instant_str.values()]), 'Barème salarié {} ne commence pas à la bonne date (il devrait en {})'.format(
                    cotisation_salarie,
                    start_non_null_date,
                    )
                assert min(start_thresholds_by_instant_str.keys()) == start_non_null_date, 'Barème salarié {} ne commence pas en {}'.format(
                    cotisation_salarie,
                    start_non_null_date,
                    )



from openfisca_france.model.base import *  # noqa analysis:ignore


class garantie_jeunes_neet(Variable):
    value_type = bool
    entity = Individu
    definition_period = MONTH
    label = 'Variable NEET - Ni étudiant, ni employé, ni stagiaire'
    set_input = set_input_divide_by_period
    reference = ['https://fr.wikipedia.org/wiki/NEET']

    def formula_2017_01_01(individu, period):
        not_in_employment = individu('salaire_net', period) == 0

        activite = individu('activite', period)
        not_in_education = (activite != TypesActivite.etudiant) * (activite != TypesActivite.actif)

        no_indemnites_stage = individu('indemnites_stage', period) == 0
        no_revenus_stage_formation_pro = individu('revenus_stage_formation_pro', period) == 0
        not_in_training = no_indemnites_stage * no_revenus_stage_formation_pro

        return not_in_employment * not_in_education * not_in_training


class garantie_jeunes_montant(Variable):
    value_type = float
    entity = Individu
    definition_period = MONTH
    label = "Montant maximal de l'allocation Garantie Jeune"
    set_input = set_input_dispatch_by_period
    reference = [
        'https://travail-emploi.gouv.fr/emploi/mesures-jeunes/garantiejeunes/',
        'https://www.service-public.fr/particuliers/vosdroits/F32700'
        ]

    def formula_2017_01_01(individu, period, parameters):
        params = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rsa
        montant_base = params.rsa_m.montant_de_base_du_rsa
        taux_1_personne = params.rsa_fl.forfait_logement.taux_1_personne
        garantie_jeunes_max = montant_base * (1 - taux_1_personne)
        salaire_minimum = parameters(period).marche_travail.salaire_minimum
        smic_mensuel_brut = salaire_minimum.smic.smic_b_horaire * salaire_minimum.smic.nb_heures_travail_mensuel

        degressivite = parameters(period).prestations_sociales.aides_jeunes.garantie_jeunes.degressivite
        plafond = degressivite.plafond * smic_mensuel_brut
        seuil_degressivite = degressivite.seuil

        types_revenus_activites = [
            'revenus_stage_formation_pro',
            'indemnites_journalieres',
            'chomage_net',
            'indemnites_volontariat',
            'asi',
            'pensions_alimentaires_percues',
            'rente_accident_travail',
            'stage_gratification',
            'bourse_enseignement_sup',
            'salaire_net',
            'bourse_recherche',
            'rpns_auto_entrepreneur_benefice',
            ]

        base_ressource = (
            sum(individu(type_revenu, period) for type_revenu in types_revenus_activites)
            )
        return garantie_jeunes_max * min_(1, max_(0, (base_ressource - plafond) / (seuil_degressivite - plafond)))


class garantie_jeunes_eligibilite_age(Variable):
    value_type = bool
    entity = Individu
    definition_period = MONTH
    label = "Éligibilité en fonction de l'âge à la Garantie Jeune"
    set_input = set_input_dispatch_by_period

    def formula_2017_01_01(individu, period, parameters):
        params_age = parameters(period).prestations_sociales.aides_jeunes.garantie_jeunes.critere_age
        age = individu('age', period)

        return (params_age.minimum <= age) * (age <= params_age.maximum)


class garantie_jeunes_eligibilite_ressources(Variable):
    value_type = bool
    entity = Individu
    definition_period = MONTH
    label = 'Éligibilité en fonction du niveau de ressources à la Garantie Jeune'
    set_input = set_input_dispatch_by_period

    def formula_2017_01_01(individu, period, parameters):
        three_previous_months = period.last_3_months
        params = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rsa
        montant_base = params.rsa_m.montant_de_base_du_rsa
        taux_1_personne = params.rsa_fl.forfait_logement.taux_1_personne
        plafond_condition_ressources = montant_base * (1 - taux_1_personne)

        ressources_individuelles = [
            'revenus_stage_formation_pro',
            'indemnites_chomage_partiel',
            'indemnites_volontariat',
            'asi',
            'pensions_alimentaires_percues',
            'stage_gratification',
            'bourse_enseignement_sup',
            'salaire_net',
            'indemnites_journalieres',
            'prestation_compensatoire',
            'rente_accident_travail',
            'pensions_invalidite',
            'aah',
            'remuneration_apprenti',
            'chomage_net',  # A éclaircir : cette ressource n'est pas mentionné dans la liste des ressources figurant dans la loi, mais plusieurs sites mentionnent leur prise en compte (dont service-public.fr, site de pole emploi)
            ]

        # Calcul sur les trois derniers mois (normalement c'est le niveau de ressources moyen le plus faible entre les 3 derniers mois et les 6 derniers mois)
        niveau_ressources_individuelles_3_mois = sum(
            individu(ressources_incluses, three_previous_months, options = [ADD]) for ressources_incluses in ressources_individuelles
            )

        rsa = individu.famille('rsa', three_previous_months, options = [ADD])
        ppa = individu.famille('ppa', three_previous_months, options = [ADD])
        rsa_ppa_demandeurs = (
            (rsa + ppa)
            * (individu.has_role(Famille.DEMANDEUR) + individu.has_role(Famille.CONJOINT))
            )

        niveau_ressources = (niveau_ressources_individuelles_3_mois + rsa_ppa_demandeurs) / 3

        return (niveau_ressources <= plafond_condition_ressources)


class garantie_jeunes(Variable):
    value_type = float
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    label = 'Montant de la Garantie Jeune'
    reference = ['https://travail-emploi.gouv.fr/emploi/mesures-jeunes/garantiejeunes/', 'https://www.service-public.fr/particuliers/vosdroits/F32700']
    end = '2022-02-28'

    def formula_2017_01_01(individu, period, parameters):
        montant = individu('garantie_jeunes_montant', period)
        neet = individu('garantie_jeunes_neet', period)
        age_ok = individu('garantie_jeunes_eligibilite_age', period)
        garantie_jeunes_eligibilite_ressources = individu('garantie_jeunes_eligibilite_ressources', period)
        return montant * neet * age_ok * garantie_jeunes_eligibilite_ressources



from numpy.core.defchararray import startswith

from openfisca_france.model.base import *


class coloc(Variable):
    value_type = bool
    entity = Menage
    label = 'Vie en colocation'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class logement_crous(Variable):
    value_type = bool
    entity = Menage
    label = 'Le logement est géré par les CROUS '
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class logement_chambre(Variable):
    value_type = bool
    entity = Menage
    label = 'Le logement est considéré comme une chambre'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class loyer(Variable):
    value_type = float
    entity = Menage
    set_input = set_input_divide_by_period
    unit = 'currency'
    label = "Loyer ou mensualité d'emprunt pour un primo-accédant"
    definition_period = MONTH


class depcom(Variable):
    value_type = str
    max_length = 5
    entity = Menage
    label = 'Code INSEE (depcom) de la commune du lieu de résidence'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesCodeInseeRegion(Enum):
    __order__ = 'non_renseigne guadeloupe martinique guyane reunion mayotte ile_de_france centre_val_de_loire bourgogne_franche_comte normandie hauts_de_france grand_est pays_de_la_loire bretagne nouvelle_aquitaine occitanie auvergne_rhone_alpes provence_alpes_cote_d_azur corse'
    non_renseigne = 'Non renseigné'
    guadeloupe = '01'
    martinique = '02'
    guyane = '03'
    reunion = '04'
    mayotte = '06'
    ile_de_france = '11'
    centre_val_de_loire = '24'
    bourgogne_franche_comte = '27'
    normandie = '28'
    hauts_de_france = '32'
    grand_est = '44'
    pays_de_la_loire = '52'
    bretagne = '53'
    nouvelle_aquitaine = '75'
    occitanie = '76'
    auvergne_rhone_alpes = '84'
    provence_alpes_cote_d_azur = '93'
    corse = '94'


class region(Variable):
    value_type = Enum
    possible_values = TypesCodeInseeRegion
    default_value = TypesCodeInseeRegion.non_renseigne
    entity = Menage
    label = 'Code INSEE de la région du lieu de résidence'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        regions_list: list[str] = parameters(period).geopolitique.regions.liste
        regions = [
            (parameters(period).geopolitique.regions[region], TypesCodeInseeRegion[region])
            for region in regions_list
            ]

        regions_elig = [
            sum([startswith(depcom, str.encode(codes_insee)) for codes_insee in parametres_region.departements]) > 0
            for (parametres_region, _) in regions
            ]
        codes_insee_regions = [code_insee_region for (_, code_insee_region) in regions]

        return select(regions_elig, codes_insee_regions, default = TypesCodeInseeRegion.non_renseigne)


class charges_locatives(Variable):
    value_type = float
    entity = Menage
    set_input = set_input_divide_by_period
    label = 'Charges locatives'
    definition_period = MONTH


class proprietaire_proche_famille(Variable):
    value_type = bool
    entity = Famille
    label = 'Le propriétaire du logement a un lien de parenté avec la personne de référence ou son conjoint'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class habite_chez_parents(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu habite chez ses parents"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class statut_occupation_logement(Variable):
    value_type = Enum
    possible_values = TypesStatutOccupationLogement  # defined in model/base.py
    entity = Menage
    default_value = TypesStatutOccupationLogement.non_renseigne
    label = "Statut d'occupation du logement"
    set_input = set_input_dispatch_by_period
    definition_period = MONTH


class residence_ile_de_france(Variable):
    label = 'Le logement est situé dans la région Île-de-France'
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return sum([startswith(depcom, str.encode(departement_idf)) for departement_idf in parameters(period).geopolitique.regions.ile_de_france.departements])  # TOOPTIMIZE: string encoding into bytes array should be done at load time


class residence_dom(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period):
        residence_guadeloupe = menage('residence_guadeloupe', period)
        residence_martinique = menage('residence_martinique', period)
        residence_guyane = menage('residence_guyane', period)
        residence_reunion = menage('residence_reunion', period)
        residence_mayotte = menage('residence_mayotte', period)

        return residence_guadeloupe + residence_martinique + residence_reunion + residence_guyane + residence_mayotte


class residence_guadeloupe(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'971')


class residence_martinique(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'972')


class residence_guyane(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'973')


class residence_reunion(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'974')


class residence_saint_pierre_et_miquelon(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'975')


class residence_mayotte(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'976')


class residence_saint_bartelemy(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'977')


class residence_saint_martin(Variable):
    value_type = bool
    entity = Menage
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        depcom = menage('depcom', period)
        return startswith(depcom, b'978')


class TypesLieuResidence(Enum):
    non_renseigne = 'Non renseigné'
    metropole = 'Métropole'
    guadeloupe = 'Guadeloupe'
    martinique = 'Martinique'
    guyane = 'Guyane'
    la_reunion = 'La réunion'
    saint_pierre_et_miquelon = 'Saint Pierre et Miquelon'
    mayotte = 'Mayotte'
    saint_bartelemy = 'Saint Bartelemy'
    saint_martin = 'Saint Martin'


class residence(Variable):
    value_type = Enum
    possible_values = TypesLieuResidence
    default_value = TypesLieuResidence.non_renseigne
    entity = Menage
    label = 'Zone de résidence'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(menage, period, parameters):
        return select(
            [
                menage('residence_guadeloupe', period),
                menage('residence_martinique', period),
                menage('residence_guyane', period),
                menage('residence_reunion', period),
                menage('residence_saint_pierre_et_miquelon', period),
                menage('residence_mayotte', period),
                menage('residence_saint_bartelemy', period),
                menage('residence_saint_martin', period)
                ],
            [
                TypesLieuResidence.guadeloupe,
                TypesLieuResidence.martinique,
                TypesLieuResidence.guyane,
                TypesLieuResidence.la_reunion,
                TypesLieuResidence.saint_pierre_et_miquelon,
                TypesLieuResidence.mayotte,
                TypesLieuResidence.saint_bartelemy,
                TypesLieuResidence.saint_martin
                ],
            default=TypesLieuResidence.metropole
            )



#! /usr/bin/env python
'''
Download tests from Ludwig (https://mes-aides.gouv.fr/tests/).

Usage examples:
    python openfisca_france/scripts/download_mes_aides_tests.py

    python openfisca_france/scripts/download_mes_aides_tests.py --test-ids xxx
    python openfisca_france/scripts/download_mes_aides_tests.py --test-ids xxx yyy ...

    python openfisca_france/scripts/download_mes_aides_tests.py --api-base-url http://localhost:9000
'''


import argparse
import json
import logging
import os
import sys

import requests
import yaml


# Globals


app_name = os.path.splitext(os.path.basename(__file__))[0]
log = logging.getLogger(app_name)

source_file_dir_name = os.path.dirname(os.path.abspath(__file__))
tests_dir_path = os.path.abspath(os.path.join(source_file_dir_name, '..', 'tests', 'mes-aides.gouv.fr'))


# Functions


def fetch_json(url):
    r = requests.get(url, headers = {'User-Agent': 'OpenFisca-Script {}'.format(app_name)})
    return r.json()


def fetch_situation(api_base_url, situation_id):
    url = '{}/api/situations/{}/openfisca-request'.format(api_base_url, situation_id)
    log.info('fetch situation: GET "{}"'.format(url))
    return fetch_json(url)


def fetch_tests(api_base_url):
    url = '{}/api/public/acceptance-tests'.format(api_base_url)
    log.info('fetch tests: GET "{}"'.format(url))
    return fetch_json(url)


def iter_yaml_items(api_base_url, test):
    test_case = fetch_situation(api_base_url = api_base_url, situation_id = test['scenario']['situationId'])
    log.info('ID: {} [{}] {}'.format(test['_id'], ', '.join(sorted(test['keywords'])), test['name']))
    yield 'name', test['name'], None
    yield 'keywords', sorted(test['keywords']), None
    description = test.get('description', None)
    description_style = '|' if description is not None and len(description.split('\n')) > 1 else None
    yield 'description', description, description_style
    relative_error_margin = {
        'accepted-exact': 0.,
        'accepted-2pct': 0.02,
        'accepted-10pct': 0.1,
        }[test['currentStatus']]
    if relative_error_margin > 0:
        yield 'relative_error_margin', relative_error_margin, None
    yield 'individus', test_case['individus'], None
    yield 'familles', test_case['familles'], None
    yield 'foyers_fiscaux', test_case['foyers_fiscaux'], None
    yield 'menages', test_case['menages'], None
    yield 'output_variables', {
        expected_result['code']: expected_result['expectedValue']
        for expected_result in test['expectedResults']
        }, None


def main():
    parser = argparse.ArgumentParser(description = __doc__)
    parser.add_argument('--api-base-url', default = 'https://mes-aides.gouv.fr', help = 'Ludwig API base URL')
    parser.add_argument('--output-dir', default = tests_dir_path, help = 'Where to write the tests')
    parser.add_argument('--tests-json', default = None, help = 'Do not download tests, use given file')
    parser.add_argument('--test-ids', default = None, help = 'Download only those IDs', nargs = '+')
    parser.add_argument('-v', '--verbose', action = 'store_true', default = False, help = 'Increase output verbosity')
    args = parser.parse_args()
    logging.basicConfig(level = logging.DEBUG if args.verbose else logging.WARNING, stream = sys.stdout)
    logging.getLogger('requests').setLevel(logging.WARNING)

    if not os.path.isdir(args.output_dir):
        os.makedirs(args.output_dir)

    existing_yaml_files_name = set(os.listdir(args.output_dir))

    if args.tests_json is None:
        tests_json = fetch_tests(api_base_url = args.api_base_url)
    else:
        log.info('load tests JSON file "{}"'.format(args.tests_json))
        with open(args.tests_json) as tests_json_file:
            tests_json_str = tests_json_file.read()
            tests_json = json.loads(tests_json_str)

    for test_json in tests_json:
        if args.test_ids is not None and test_json['_id'] not in args.test_ids:
            continue

        assert test_json['currentStatus'] in ('accepted-exact', 'accepted-2pct', 'accepted-10pct', 'rejected')
        if test_json['currentStatus'] == 'rejected':
            return
        last_execution = test_json['lastExecution']
        assert test_json['currentStatus'] == last_execution['status']
        if any('expectedValue' in result and 'status' in result and result['status'] not in ('accepted-exact', 'accepted-2pct') for result in last_execution['results']):
            log.info('Test "{}" doesn\'t return the expected value (yet), so skip it.'.format(test_json['_id']))
            continue

        # Write test_yaml to output_dir_path
        yaml_file_name = 'test_mes_aides_{}.yaml'.format(test_json['_id'])
        yaml_file_path = os.path.join(args.output_dir, yaml_file_name)
        with open(yaml_file_path, 'w') as yaml_file:
            for yaml_key, yaml_value, default_style in iter_yaml_items(api_base_url = args.api_base_url,
                    test = test_json):
                yaml.safe_dump({yaml_key: yaml_value}, yaml_file, allow_unicode = True, encoding = 'utf-8',
                    default_flow_style = False, default_style = default_style, indent = 2, width = 120)

        if yaml_file_name in existing_yaml_files_name:
            existing_yaml_files_name.remove(yaml_file_name)

        # Verify YAML syntax of generated file.
        with open(yaml_file_path, 'r') as yaml_file:
            scenario = yaml.load(yaml_file)
        assert isinstance(scenario, dict), (yaml_file_path, scenario, test_json)

    if args.test_ids is None:
        for file_name in sorted(existing_yaml_files_name):
            log.info('Deleting obsolete test: "{}"'.format(file_name))
            file_path = os.path.join(args.output_dir, file_name)
            os.remove(file_path)

    return 0


if __name__ == '__main__':
    sys.exit(main())



from openfisca_france.model.base import *  # noqa analysis:ignore


class pass_culture(Variable):
    value_type = float
    entity = Individu
    definition_period = MONTH
    label = 'Montant du pass culture'
    set_input = set_input_divide_by_period
    reference = ['https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000043518870', 'https://www.service-public.fr/particuliers/vosdroits/F34959']

    def formula(individu, period, parameters):
        montant = parameters(period).prestations_sociales.aides_jeunes.pass_culture.montants
        age = individu('age', period.offset(1, 'month'))
        age_maximum = parameters(period).prestations_sociales.aides_jeunes.pass_culture.age_maximum
        return montant.calc(age) * (age <= age_maximum)



import logging

from openfisca_france.model.base import *

# TODO:
# check hsup everywhere !

# Helpers

from openfisca_france.model.prelevements_obligatoires.prelevements_sociaux.cotisations_sociales.base import apply_bareme

log = logging.getLogger(__name__)

taux_aot_by_depcom = None
taux_smt_by_depcom = None


# Cotisations proprement dites


class conge_individuel_formation_cdd(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution au financement des congé individuel de formation (CIF) des salariées en CDD'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    # TODO: date de début
    def formula(individu, period, parameters):
        contrat_de_travail_type = individu('contrat_de_travail_type', period)

        TypesContrat = contrat_de_travail_type.possible_values
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)

        conge_individuel_formation = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.formation.toute_entreprise
        cotisation = - conge_individuel_formation.cdd * (contrat_de_travail_type == TypesContrat.cdd) * assiette_cotisations_sociales

        return cotisation


class redevable_taxe_apprentissage(Variable):
    value_type = bool
    entity = Individu
    label = "Entreprise redevable de la taxe d'apprentissage"
    reference = 'https://www.economie.gouv.fr/entreprises/taxe-apprentissage'  # TODO: Coder les exonerations selon la masse salariale
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        # L'association a but non lucratif ne paie pas d'IS de droit commun article 206 du Code général des impôts
        # -> pas de taxe d'apprentissage
        association = individu('entreprise_est_association_non_lucrative', period)

        return not_(association)


class contribution_developpement_apprentissage(Variable):
    value_type = float
    entity = Individu
    label = "Contribution additionnelle au développement de l'apprentissage"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        redevable_taxe_apprentissage = individu('redevable_taxe_apprentissage', period)

        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'apprentissage_contribution_additionnelle',
            variable_name = 'contribution_developpement_apprentissage',
            )
        return cotisation * redevable_taxe_apprentissage


class contribution_supplementaire_apprentissage(Variable):
    value_type = float
    entity = Individu
    label = "Contribution supplémentaire à l'apprentissage"
    reference = 'https://www.service-public.fr/professionnels-entreprises/vosdroits/F22574'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2015_01_01(individu, period, parameters):
        redevable_taxe_apprentissage = individu('redevable_taxe_apprentissage', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        ratio_alternants = individu('ratio_alternants', period)
        effectif_entreprise = individu('effectif_entreprise', period)
        salarie_regime_alsace_moselle = individu('salarie_regime_alsace_moselle', period)
        contribution = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.apprentissage.csa

        multiplier = (salarie_regime_alsace_moselle * contribution.multiplicateur_alsace_moselle) + (1 - salarie_regime_alsace_moselle)

        taxe_due = (effectif_entreprise >= 250) * (ratio_alternants < .05)
        taux_conditionnel = (
            (effectif_entreprise >= 2000) * (ratio_alternants < .01) * contribution.plus_de_2000_moins_de_1pc
            + (effectif_entreprise < 2000) * (ratio_alternants < .01) * contribution.plus_de_250_moins_de_1pc
            + (.01 <= ratio_alternants) * (ratio_alternants < .02) * contribution.plus_de_250_entre_1_et_2pc
            + (.02 <= ratio_alternants) * (ratio_alternants < .03) * contribution.plus_de_250_entre_2_et_3pc
            + (.03 <= ratio_alternants) * (ratio_alternants < .04) * contribution.plus_de_250_entre_3_et_4pc
            + (.04 <= ratio_alternants) * (ratio_alternants < .05) * contribution.plus_de_250_entre_4_et_5pc
            )
        taux_contribution = taxe_due * taux_conditionnel * multiplier
        return - taux_contribution * assiette_cotisations_sociales * redevable_taxe_apprentissage

    def formula_2012_01_01(individu, period, parameters):
        redevable_taxe_apprentissage = individu('redevable_taxe_apprentissage', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        effectif_entreprise = individu('effectif_entreprise', period)
        ratio_alternants = individu('ratio_alternants', period)
        salarie_regime_alsace_moselle = individu('salarie_regime_alsace_moselle', period)
        contribution = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.apprentissage.csa

        multiplier = (salarie_regime_alsace_moselle * contribution.multiplicateur_alsace_moselle) + (1 - salarie_regime_alsace_moselle)

        taxe_due = (effectif_entreprise >= 250) * (ratio_alternants < .04)
        taux_conditionnel = (
            (effectif_entreprise >= 2000) * (ratio_alternants < .01) * contribution.plus_de_2000_moins_de_1pc
            + (effectif_entreprise < 2000) * (ratio_alternants < .01) * contribution.plus_de_250_moins_de_1pc
            + (.01 <= ratio_alternants) * (ratio_alternants < .02) * contribution.plus_de_250_entre_1_et_2pc
            + (.02 <= ratio_alternants) * (ratio_alternants < .03) * contribution.plus_de_250_entre_2_et_3pc
            + (.03 <= ratio_alternants) * (ratio_alternants < .04) * contribution.plus_de_250_entre_3_et_4pc
            )
        taux_contribution = taux_conditionnel * taxe_due * multiplier
        return - taux_contribution * assiette_cotisations_sociales * redevable_taxe_apprentissage

    def formula_2011_01_01(individu, period, parameters):
        redevable_taxe_apprentissage = individu('redevable_taxe_apprentissage', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        effectif_entreprise = individu('effectif_entreprise', period)
        ratio_alternants = individu('ratio_alternants', period)
        contribution = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.apprentissage.csa

        taxe_due = (effectif_entreprise >= 250) * (ratio_alternants < .04)
        taux_conditionnel = (
            (effectif_entreprise >= 2000) * (ratio_alternants < .01) * contribution.plus_de_2000_moins_de_1pc
            + (effectif_entreprise < 2000) * (ratio_alternants < .01) * contribution.plus_de_250_moins_de_1pc
            + (.01 <= ratio_alternants) * (ratio_alternants < .02) * contribution.plus_de_250_entre_1_et_2pc
            + (.02 <= ratio_alternants) * (ratio_alternants < .03) * contribution.plus_de_250_entre_2_et_3pc
            + (.03 <= ratio_alternants) * (ratio_alternants < .04) * contribution.plus_de_250_entre_3_et_4pc
            )
        taux_contribution = taux_conditionnel * taxe_due
        return - taux_contribution * assiette_cotisations_sociales * redevable_taxe_apprentissage

    def formula_2009_01_01(individu, period, parameters):
        redevable_taxe_apprentissage = individu('redevable_taxe_apprentissage', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        effectif_entreprise = individu('effectif_entreprise', period)
        ratio_alternants = individu('ratio_alternants', period)
        contribution = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.apprentissage.csa

        taxe_due = (effectif_entreprise >= 250) * (ratio_alternants < .03)
        taux_conditionnel = (
            (ratio_alternants < .01) * contribution.plus_de_250_moins_de_1pc
            + (.01 <= ratio_alternants) * (ratio_alternants < .02) * contribution.plus_de_250_entre_1_et_2pc
            + (.02 <= ratio_alternants) * (ratio_alternants < .03) * contribution.plus_de_250_entre_2_et_3pc
            )
        taux_contribution = taux_conditionnel * taxe_due
        return - taux_contribution * assiette_cotisations_sociales * redevable_taxe_apprentissage


class cotisations_employeur_main_d_oeuvre(Variable):
    value_type = float
    entity = Individu
    label = "Cotisation sociales employeur main d'oeuvre"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_01_01(individu, period, parameters):
        # En 2019, la taxe d'apprentissage et la CFP sont regroupées dans la CUFPA
        conge_individuel_formation_cdd = individu('conge_individuel_formation_cdd', period)
        contribution_developpement_apprentissage = individu(
            'contribution_developpement_apprentissage', period)
        contribution_supplementaire_apprentissage = individu(
            'contribution_supplementaire_apprentissage', period)
        financement_organisations_syndicales = individu('financement_organisations_syndicales', period)
        fnal = individu('fnal', period)
        participation_effort_construction = individu('participation_effort_construction', period, options = [ADD])
        prevoyance_obligatoire_cadre = individu('prevoyance_obligatoire_cadre', period, options = [ADD])
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options = [ADD])
        complementaire_sante_employeur = individu('complementaire_sante_employeur', period, options = [ADD])
        versement_transport = individu('versement_transport', period, options = [ADD])

        contribution_unique_formation_professionnelle_alternance = individu('contribution_unique_formation_professionnelle_alternance', period, options = [ADD])

        cotisations_employeur_main_d_oeuvre = (
            conge_individuel_formation_cdd
            + contribution_developpement_apprentissage
            + contribution_supplementaire_apprentissage
            + financement_organisations_syndicales
            + fnal
            + participation_effort_construction
            + prevoyance_obligatoire_cadre
            + prevoyance_complementaire_employeur
            + complementaire_sante_employeur
            + versement_transport
            + contribution_unique_formation_professionnelle_alternance
            )

        return cotisations_employeur_main_d_oeuvre

    def formula(individu, period, parameters):
        conge_individuel_formation_cdd = individu('conge_individuel_formation_cdd', period)
        contribution_developpement_apprentissage = individu(
            'contribution_developpement_apprentissage', period)
        contribution_supplementaire_apprentissage = individu(
            'contribution_supplementaire_apprentissage', period)
        financement_organisations_syndicales = individu('financement_organisations_syndicales', period)
        fnal = individu('fnal', period)
        formation_professionnelle = individu('formation_professionnelle', period)
        participation_effort_construction = individu('participation_effort_construction', period, options = [ADD])
        prevoyance_obligatoire_cadre = individu('prevoyance_obligatoire_cadre', period, options = [ADD])
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options = [ADD])
        complementaire_sante_employeur = individu('complementaire_sante_employeur', period, options = [ADD])

        taxe_apprentissage = individu('taxe_apprentissage', period, options = [ADD])
        versement_transport = individu('versement_transport', period, options = [ADD])

        cotisations_employeur_main_d_oeuvre = (
            conge_individuel_formation_cdd
            + contribution_developpement_apprentissage
            + contribution_supplementaire_apprentissage
            + financement_organisations_syndicales
            + fnal
            + formation_professionnelle
            + participation_effort_construction
            + prevoyance_obligatoire_cadre
            + prevoyance_complementaire_employeur
            + complementaire_sante_employeur
            + taxe_apprentissage
            + versement_transport
            )

        return cotisations_employeur_main_d_oeuvre


class fnal(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation fonds national action logement (FNAL)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        fnal_cotisation = individu('fnal_cotisation', period)
        fnal_contribution = individu('fnal_contribution', period)
        return fnal_cotisation + fnal_contribution


class fnal_cotisation(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation fonds national action logement (FNAL)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2015-01-01'

    def formula(individu, period, parameters):
        return apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'fnal_cotisation',
            variable_name = 'fnal_cotisation',
            )


class fnal_contribution(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution fonds national action logement (FNAL)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2020_01_01(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)
        effectif_plus_de_50_salaries = effectif_entreprise >= 50
        contribution_plus_de_50_salaries = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'fnal_contribution_plus_de_50_salaries',
            variable_name = 'fnal_contribution',
            )
        contribution_moins_de_50_salaries = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'fnal_contribution_moins_de_50_salaries',
            variable_name = 'fnal_contribution',
            )
        return effectif_plus_de_50_salaries * contribution_plus_de_50_salaries + (1 - effectif_plus_de_50_salaries) * contribution_moins_de_50_salaries

    def formula_2015_01_01(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)
        effectif_plus_de_20_salaries = effectif_entreprise >= 20
        contribution_plus_de_20_salaries = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'fnal_contribution_plus_de_20_salaries',
            variable_name = 'fnal_contribution',
            )
        contribution_moins_de_20_salaries = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'fnal_contribution_moins_de_20_salaries',
            variable_name = 'fnal_contribution',
            )
        return effectif_plus_de_20_salaries * contribution_plus_de_20_salaries + (1 - effectif_plus_de_20_salaries) * contribution_moins_de_20_salaries

    def formula_2007_01_01(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)
        effectif_plus_de_20_salaries = effectif_entreprise >= 20
        contribution_plus_de_20_salaries = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'fnal_contribution_plus_de_20_salaries',
            variable_name = 'fnal_contribution',
            )
        return effectif_plus_de_20_salaries * contribution_plus_de_20_salaries

    def formula(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)
        effectif_plus_de_10_salaries = effectif_entreprise >= 10
        contribution_plus_de_10_salaries = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'fnal_contribution_plus_de_10_salaries',
            variable_name = 'fnal_contribution',
            )
        return effectif_plus_de_10_salaries * contribution_plus_de_10_salaries


class financement_organisations_syndicales(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution patronale au financement des organisations syndicales'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2015_01_01(individu, period, parameters):
        categorie_salarie = individu('categorie_salarie', period)
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'financement_organisations_syndicales',
            variable_name = 'financement_organisations_syndicales',
            )
        contrat_de_droit_prive = (
            + (categorie_salarie == TypesCategorieSalarie.prive_non_cadre)
            + (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            + (categorie_salarie == TypesCategorieSalarie.public_non_titulaire)
            )

        return cotisation * contrat_de_droit_prive


class formation_professionnelle(Variable):
    value_type = float
    entity = Individu
    label = 'PEFPC - Formation professionnelle'
    reference = 'https://www.service-public.fr/professionnels-entreprises/vosdroits/F22570'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'  # Remplacé par la contribution_formation_professionnelle

    def formula_2016_01_01(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)

        cotisation_0_10 = (effectif_entreprise < 11) * apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'formprof_moins_de_11_salaries',
            variable_name = 'formation_professionnelle',
            )

        cotisation_11 = (effectif_entreprise >= 11) * apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'formprof_11_salaries_et_plus',
            variable_name = 'formation_professionnelle',
            )
        return cotisation_0_10 + cotisation_11

    def formula(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)

        cotisation_0_9 = (effectif_entreprise < 10) * apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'formprof_moins_de_10_salaries',
            variable_name = 'formation_professionnelle',
            )

        cotisation_10_19 = ((effectif_entreprise >= 10) * (effectif_entreprise < 20)) * apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'formprof_entre_10_et_19_salaries',
            variable_name = 'formation_professionnelle',
            )

        cotisation_20 = (effectif_entreprise >= 20) * apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'formprof_20_salaries_et_plus',
            variable_name = 'formation_professionnelle',
            )
        return cotisation_0_9 + cotisation_10_19 + cotisation_20


class contribution_formation_professionnelle(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution à la formation professionnelle (CFP)'
    reference = 'https://www.urssaf.fr/portail/home/espaces-dedies/contributions-de-formation-profe/la-contribution-a-la-formation-p.html'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # Initiée en 2019: avec la Taxe d'apprentissage, elles forment la CUFPA (contribution unique à la formation professionnelle et à l'alternance)

    def formula_2019_01_01(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)
        apprenti = individu('apprenti', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        contribution = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.contribution_unique_formation.contrib_formation_pro

        taux_contribution = ((effectif_entreprise >= 11) * contribution.onze_et_plus_salaries
            + (effectif_entreprise < 11) * not_(apprenti) * contribution.moins_11_salaries)

        return - taux_contribution * assiette_cotisations_sociales


class participation_effort_construction(Variable):
    value_type = float
    entity = Individu
    label = "Participation à l'effort de construction"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # TO DO : integration de la variable peec_employeur : les critères d'éligibilité employeur incluent d'autres dimensions que l'effectif

    def formula_2020_01_01(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)

        bareme = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'construction_plus_de_50_salaries',
            variable_name = 'participation_effort_construction',
            )

        cotisation = bareme * (effectif_entreprise >= 50)

        return cotisation

    def formula_2005_01_01(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)

        bareme = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'construction_plus_de_20_salaries',
            variable_name = 'participation_effort_construction',
            )

        cotisation = bareme * (effectif_entreprise >= 20)

        return cotisation

    def formula(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)

        bareme = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'construction_plus_de_10_salaries',
            variable_name = 'participation_effort_construction',
            )

        cotisation = bareme * (effectif_entreprise >= 10)

        return cotisation


class taxe_apprentissage(Variable):
    value_type = float
    entity = Individu
    label = "Taxe d'apprentissage (employeur, entreprise redevable de la taxe d'apprentissage uniquement)"
    reference = 'https://www.service-public.fr/professionnels-entreprises/vosdroits/F22574'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        redevable_taxe_apprentissage = individu('redevable_taxe_apprentissage', period)
        salarie_regime_alsace_moselle = individu('salarie_regime_alsace_moselle', period)

        cotisation_regime_alsace_moselle = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'apprentissage_taxe_alsace_moselle',
            variable_name = 'taxe_apprentissage',
            )

        cotisation_regime_general = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'apprentissage_taxe',
            variable_name = 'taxe_apprentissage',
            )

        cotisation = where(
            salarie_regime_alsace_moselle,
            cotisation_regime_alsace_moselle,
            cotisation_regime_general,
            )

        # En 2019 on a une année blanche, mais la formule et le taux sont inchangés
        if period.start.year == 2019:
            cotisation = 0

        return cotisation * redevable_taxe_apprentissage


class contribution_unique_formation_professionnelle_alternance(Variable):
    value_type = float
    entity = Individu
    label = "Contribution Unique à la Formation Professionnelle et à l'Alternance (CUFPA)"
    definition_period = MONTH
    reference = 'https://www.legifrance.gouv.fr/conv_coll/id/KALIARTI000041698592/?idConteneur=KALICONT000042181870'
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        contribution_formation_professionnelle = individu('contribution_formation_professionnelle', period)
        taxe_apprentissage = individu('taxe_apprentissage', period, options = [ADD])

        return contribution_formation_professionnelle + taxe_apprentissage


class taxe_salaires(Variable):
    value_type = float
    entity = Individu
    label = 'Taxe sur les salaires'
    reference = 'http://www.impots.gouv.fr/portal/deploiement/p1/fichedescriptiveformulaire_8920/fichedescriptiveformulaire_8920.pdf'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2018_01_01(individu, period, parameters):
        assujettie_taxe_salaires = individu('assujettie_taxe_salaires', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        complementaire_sante_employeur = individu('complementaire_sante_employeur', period)
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options = [ADD])

        entreprise_est_association_non_lucrative = individu('entreprise_est_association_non_lucrative', period)
        effectif_entreprise = individu('effectif_entreprise', period)

        # impots.gouv.fr
        # La taxe est due notamment par les : [...] organismes sans but lucratif
        assujettissement = assujettie_taxe_salaires + entreprise_est_association_non_lucrative

        taxe_salaires = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.taxsal
        bareme = taxe_salaires.taux_maj
        base = assiette_cotisations_sociales + (
            prevoyance_complementaire_employeur
            - complementaire_sante_employeur
            )

        # TODO: exonérations apprentis
        # TODO: modify if DOM

        cotisation_individuelle = (
            bareme.calc(
                base,
                factor = 1 / 12,
                round_base_decimals = 2
                )
            + round_(taxe_salaires.metro * base, 2)
            )

        # Une franchise et une décôte s'appliquent à cette taxe
        # Etant donné que nous n'avons pas la distribution de salaires de l'entreprise,
        # elles sont estimées en prenant l'effectif de l'entreprise et
        # considérant que l'unique salarié de la individu est la moyenne.
        # http://www.impots.gouv.fr/portal/dgi/public/popup?typePage=cpr02&espId=2&docOid=documentstandard_1845
        estimation = cotisation_individuelle * effectif_entreprise * 12
        conditions = [estimation < taxe_salaires.franchise, estimation <= taxe_salaires.decote_montant, estimation > taxe_salaires.decote_montant]
        results = [0, estimation - (taxe_salaires.decote_montant - estimation) * taxe_salaires.decote_taux, estimation]

        estimation_reduite = select(conditions, results)

        # Abattement spécial de taxe sur les salaires
        # Les associations à but non lucratif bénéficient d'un abattement important
        estimation_abattue_negative = estimation_reduite - taxe_salaires.abattement_special
        estimation_abattue = where(entreprise_est_association_non_lucrative,
                                   (estimation_abattue_negative >= 0) * estimation_abattue_negative,
                                   estimation_reduite
                                   )

        cotisation = where(effectif_entreprise == 0,
                           individu.filled_array(0),
                           estimation_abattue / effectif_entreprise / 12
                           )

        return - cotisation * assujettissement

    def formula(individu, period, parameters):
        assujettie_taxe_salaires = individu('assujettie_taxe_salaires', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        complementaire_sante_employeur = individu('complementaire_sante_employeur', period)
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options = [ADD])
        prevoyance_obligatoire_cadre = individu('prevoyance_obligatoire_cadre', period, options = [ADD])

        entreprise_est_association_non_lucrative = individu('entreprise_est_association_non_lucrative', period)
        effectif_entreprise = individu('effectif_entreprise', period)

        # impots.gouv.fr
        # La taxe est due notamment par les : [...] organismes sans but lucratif
        assujettissement = assujettie_taxe_salaires + entreprise_est_association_non_lucrative

        taxe_salaires = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.taxsal
        bareme = taxe_salaires.taux_maj
        base = assiette_cotisations_sociales + (
            - prevoyance_obligatoire_cadre
            + prevoyance_complementaire_employeur
            - complementaire_sante_employeur
            )

        # TODO: exonérations apprentis
        # TODO: modify if DOM

        cotisation_individuelle = (
            bareme.calc(
                base,
                factor = 1 / 12,
                round_base_decimals = 2
                )
            + round_(taxe_salaires.metro * base, 2)
            )

        # Une franchise et une décôte s'appliquent à cette taxe
        # Etant donné que nous n'avons pas la distribution de salaires de l'entreprise,
        # elles sont estimées en prenant l'effectif de l'entreprise et
        # considérant que l'unique salarié de la individu est la moyenne.
        # http://www.impots.gouv.fr/portal/dgi/public/popup?typePage=cpr02&espId=2&docOid=documentstandard_1845
        estimation = cotisation_individuelle * effectif_entreprise * 12
        conditions = [estimation < taxe_salaires.franchise, estimation <= taxe_salaires.decote_montant, estimation > taxe_salaires.decote_montant]
        results = [0, estimation - (taxe_salaires.decote_montant - estimation) * taxe_salaires.decote_taux, estimation]

        estimation_reduite = select(conditions, results)

        # Abattement spécial de taxe sur les salaires
        # Les associations à but non lucratif bénéficient d'un abattement important
        estimation_abattue_negative = estimation_reduite - taxe_salaires.abattement_special
        estimation_abattue = where(entreprise_est_association_non_lucrative,
                                   (estimation_abattue_negative >= 0) * estimation_abattue_negative,
                                   estimation_reduite
                                   )

        cotisation = where(effectif_entreprise == 0,
                           individu.filled_array(0),
                           estimation_abattue / effectif_entreprise / 12
                           )

        return - cotisation * assujettissement



# When using openfisca for a large population, having too many variables in cache make openfisca performances drop.
# The following variables are intermediate results and do not need to be cached in those usecases.

cache_blacklist = set([
    'agepi_temps_travail_semaine',
    'aide_financement_permis_apprenti_eligibilite',
    'aide_permis_demandeur_emploi_eligibilite_financiere',
    'aide_permis_demandeur_emploi_eligibilite_individu',
    'aide_logement_loyer_retenu',
    'aide_logement_charges',
    'aide_logement_R0',
    'aide_logement_taux_famille',
    'aide_logement_taux_loyer',
    'aide_logement_participation_personnelle',
    'aide_logement_montant_brut_avant_degressivite',
    'aides_logement_primo_accedant',
    'aides_logement_primo_accedant_k',
    'aides_logement_nb_part',
    'aides_logement_primo_accedant_plafond_mensualite',
    'aides_logement_primo_accedant_ressources',
    'bourse_criteres_sociaux_base_ressources',
    'bourse_criteres_sociaux_base_ressources_etudiant_autonome',
    'bourse_criteres_sociaux_distance_domicile_familial',
    'bourse_criteres_sociaux_echelon',
    'bourse_criteres_sociaux_eligibilite',
    'bourse_criteres_sociaux_eligibilite_age',
    'bourse_criteres_sociaux_eligibilite_age_nombre_enfants',
    'bourse_criteres_sociaux_eligibilite_etude',
    'bourse_criteres_sociaux_etudiant_autonome',
    'bourse_criteres_sociaux_etudiant_autonome_ressource_mensuelle',
    'bourse_criteres_sociaux_nombre_enfants_a_charge',
    'bourse_criteres_sociaux_nombre_enfants_a_charge_dans_enseignement_superieur',
    'bourse_criteres_sociaux_nombre_enfants_parent_etudiant',
    'bourse_criteres_sociaux_points_de_charge',
    'bourse_criteres_sociaux_points_de_charge_charges_familiale',
    'bourse_criteres_sociaux_points_de_charge_distance_domicile_familial',
    'cheque_energie',
    'allocation_retour_emploi_journaliere_taux_plein',
    'garantie_jeunes_neet',
    'garantie_jeunes_eligibilite_age',
    'locapass_eligibilite_etudiant',
    'locapass_eligibilite_etudiant_contrat',
    'locapass_eligibilite_individu',
    'locapass_eligibilite_jeunes',
    'locapass_eligibilite_logement',
    'locapass_eligibilite_salarie',
    'montant_cheque_energie',
    'ppa_forfait_logement',
    'ppa_plancher_revenu_activite_etudiant',
    'unites_consommation_cheque_energie',
    'rsa_revenus_determination_enfant_a_charge',
    ])



from openfisca_france.model.base import *


class taux_capacite_travail(Variable):
    value_type = float
    default_value = 1.0
    entity = Individu
    label = "Taux de capacité de travail, appréciée par la commission des droits et de l'autonomie des personnes handicapées (CDAPH)"
    definition_period = MONTH
    is_period_size_independent = True
    set_input = set_input_dispatch_by_period


class taux_incapacite(Variable):
    value_type = float
    entity = Individu
    label = "Taux d'incapacité pris en compte pour l'AAH"
    definition_period = MONTH
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=BD54F4B28313142C87FC8B96013E0441.tplgfr44s_1?idArticle=LEGIARTI000023097719&cidTexte=LEGITEXT000006073189&dateTexte=20190312'
    documentation = "Taux d'incapacité pris en compte pour l'allocation adulte handicapé (AAH)."
    is_period_size_independent = True
    set_input = set_input_dispatch_by_period
    unit = '/1'
